package designer.admin;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/
 /**
  * This class provides administration functions based upon the wrapped methods in the
  * superclass.
  */
import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import com.ibm.mqe.mqbridge.*;
import java.util.*;
public class UsefulQueueManager extends WrappedQueueManager {
	private String localQueueManagerName;
/**
 * UsefulQueueManager constructor comment.
 */
public UsefulQueueManager() {
	super();
}
protected final String _getReason(MQeAdminMsg replyMsg) {
	try {
		if (null == replyMsg) {
			return "No Reply";
		}
		return replyMsg.getReason();
	} catch (Exception e) {
	}
	return "";
}
protected final int _getResultCode(MQeAdminMsg replyMsg) {
	try {
		return replyMsg.getRC();
	} catch (Exception e) {
	}
	return -1;
}
protected final boolean _isSuccess(MQeAdminMsg replyMsg) {
	return MQeAdminMsg.RC_Success == _getResultCode(replyMsg);
}
protected boolean addAliasToConnectionDefinitionRemotely(String targetQueueManagerName, String remoteQueueManagerName, String alias, boolean reportFailure) {
	// create a route to the remote queue manager
	boolean success = false;
	try {
		MQeConnectionAdminMsg msg = new MQeConnectionAdminMsg();
		msg.setName(remoteQueueManagerName);
		msg.addAlias(alias);

		MQeAdminMsg reply = processRemoteAdminMsg(targetQueueManagerName, msg, reportFailure);
		if (null != reply) {
			success = _isSuccess(reply);
		}
		if (!success && reportFailure) {
			_error("Failed to add connection alias: " + alias + " to " + remoteQueueManagerName);
			_error("Reason: " + _getReason(reply));
		}

	} catch (Exception e) {
		if (reportFailure) {
			_error("Exception adding alias: " + e.getMessage());
		}
	}
	return success;
}
protected final String addLocalAdminReplyRequestTo(MQeMsgObject msg) {
	return addReplyRequestTo(getLocalQueueManagerName(),
		MQe.Admin_Reply_Queue_Name,
		msg);
}
public final boolean addQueueAliasRemotely(String queueManagerName, String queueQueueManagerName, String queueName, String alias, boolean showErrors) {
	boolean success = false;
	
	try {
		MQeQueueAdminMsg msg = new MQeQueueAdminMsg(queueQueueManagerName, queueName);
		msg.addAlias(alias);
		MQeAdminMsg reply = processRemoteAdminMsg(queueManagerName, msg, showErrors);
		if (null != reply) {
			success = _isSuccess(reply);
		}
		if (showErrors && !success) {
			_error("Failed to add alias " + alias + " to queue: " + queueName +"@"+ queueManagerName);
			_error("Reason: " + _getReason(reply));
			checkReply(reply);
		}

	} catch (Exception e) {
		if (showErrors) {
			_error("adding alias " + alias + " to queue: " + queueName +"@"+ queueManagerName, e);
		}
			success = false;
	}
	
	return success;
}
protected boolean addQueueManagerToStoreAndForwardQueueRemotely(String destQMName, String queueQueueManagerName, String queueName, String addQmName, boolean showErrors) {
	boolean success = false;
	try {
		MQeStoreAndForwardQueueAdminMsg msg = new MQeStoreAndForwardQueueAdminMsg(queueQueueManagerName, queueName);
		MQeFields parameters = new MQeFields();
		msg.addQueueManager(addQmName);
		MQeAdminMsg reply = processRemoteAdminMsg(destQMName, msg, showErrors);
		if (null != reply) {
			success = _isSuccess(reply);
		}
		if (!success) {
			_error("Failed to add queue manager " + addQmName + " to S&F queue " + queueQueueManagerName + "+" + queueName);
			_error("Reason: " + _getReason(reply));
		}

	} catch (Exception e) {
		e.printStackTrace();
		success = false;
	}
	return success;
}
protected final String addRemoteAdminReplyRequestTo(String queueManagerName, MQeMsgObject msg) {
	return addReplyRequestTo(queueManagerName,
		MQe.Admin_Reply_Queue_Name,
		msg);
}
protected final String addReplyRequestTo(String replyQueueManagerName, String replyQueueName, MQeMsgObject msg) {
	String match = null;
	try {
		match = "Msg" + System.currentTimeMillis();
		msg.putInt(MQe.Msg_Style, MQe.Msg_Style_Request);
		msg.putAscii(MQe.Msg_ReplyToQ, replyQueueName);
		msg.putAscii(MQe.Msg_ReplyToQMgr, replyQueueManagerName);
		msg.putArrayOfByte( MQe.Msg_CorrelID, match.getBytes());
	} catch (Exception e) {
		match = null;
		_error("adding reply details to message ", e);
	}
	return match;
}
/**
 * Check to see if a good reply was required
 * If not detail the error(s) that occured
 * @return boolean  true if good, or if partly good
 * @param replyMsg reply message to check
 */
public final boolean checkReply(MQeAdminMsg replyMsg) {
	boolean reply = true;
	try {
		// Was a reply received ?
		if (replyMsg == null) {
			//_out().println("..No response received to the request");
			//throw new MQeException("No response message received");
			reply = false;
		} else if (replyMsg.getRC() != MQeAdminMsg.RC_Success) {
			reply = false;
			// If mixed then list each error that occured
			if (replyMsg.getRC() == MQeAdminMsg.RC_Mixed) {
				reply = true;
				MQeFields errors = replyMsg.getErrorFields();
				Enumeration en = errors.fields();
				// process each error
				while (en.hasMoreElements()) {
					String value[];
					String name = (String) en.nextElement();
					value = replyMsg.getFieldInError(name);
					for (int j = 0; j < value.length; j++)
						_error("Field in error: " + name + "  " + value[j]);
				}
			}
		}
	} catch (Exception e) {
		reply = false;
	}
	return reply;
}
protected boolean createAdminQueueRemotely(String queueManagerName, String queueName, MQeFields parameters, boolean reportErrors) {
	boolean success = false;
	try {
		if (null == parameters) {
			parameters = new MQeFields();
		}
		MQeAdminQueueAdminMsg msg = new MQeAdminQueueAdminMsg(queueManagerName, queueName);
		msg.create(parameters);
		MQeAdminMsg reply = processRemoteAdminMsg(queueManagerName, msg, reportErrors);
		if (null != reply) {
			success = _isSuccess(reply);
		}
		if (reportErrors && !success) {
			_error("Failed to create admin queue: " + queueManagerName + "+" + queueName);
			_error("Reason: " + _getReason(reply));
			checkReply(reply);
		}

	} catch (Exception e) {
		if (reportErrors) {
			_error("creating local queue remotely");
		}
			success = false;
	}
	return success;
}
/**
 * Create an Asynchronous remote queue and put messages
 * to it repeatedly, until key pressed.
 */
protected boolean createAsynchronousRemoteQueueRemotely(String remoteQmName, String queueQueueManagerName, String queueName, boolean reportErrors) {
	MQeFields params = new MQeFields();
	boolean success = false;
	try {
		params.putByte(MQeQueueAdminMsg.Queue_Mode, MQeQueueAdminMsg.Queue_Asynchronous);
		success = createRemoteQueueRemotely(remoteQmName, queueQueueManagerName, queueName, params, reportErrors);
	} catch (Exception e) {
		_error("Setting up local definition of remote queue");
		success = false;
	}
	return success;
}
public final boolean createBridgeConnectionRemotely(String queueManagerName, String bridgeName, String bridgeQueueManagerName, String bridgeConnectionName, boolean showErrors) {
	boolean result = false;;
	try {
		MQeClientConnectionAdminMsg msg = new MQeClientConnectionAdminMsg(bridgeName, bridgeQueueManagerName, bridgeConnectionName, false);
		msg.setAction(MQeAdminMsg.Action_Create);
		MQeAdminMsg reply = processRemoteAdminMsg(queueManagerName, msg, true);
		if (null == reply) {
			if (showErrors) {
				_error("Failed to create bridge connection  " +bridgeConnectionName+ " on " + queueManagerName + ": no reply received");
			}
		} else if (!_isSuccess(reply)) {
			if (showErrors) {
				_error("Failed to create bridge connection " +bridgeConnectionName+ " on " + queueManagerName);
				_error("Reason: " + _getReason(reply));
			}
		} else {
			result = true;
		}
	} catch (NoClassDefFoundError e) {
	} catch (Exception e) {
		_comment("exception while creating bridge connectionM : ", e);
	}
	return result;
}
public final boolean createBridgeListenerRemotely(String queueManagerName, String bridgeName, String bridgeQueueManagerName, String bridgeConnectionName, String listenerName, boolean showErrors) {
	boolean result = false;;
	try {
		MQeListenerAdminMsg msg = new MQeListenerAdminMsg(bridgeName, bridgeQueueManagerName, bridgeConnectionName, listenerName, false);
		msg.setAction(MQeAdminMsg.Action_Create);
		MQeAdminMsg reply = processRemoteAdminMsg(queueManagerName, msg, true);
		if (null == reply) {
			if (showErrors) {
				_error("Failed to create bridge listener " +listenerName+ " on " + queueManagerName + ": no reply received");
			}
		} else if (!_isSuccess(reply)) {
			if (showErrors) {
				_error("Failed to create bridge listener " +listenerName+ " on " + queueManagerName);
				_error("Reason: " + _getReason(reply));
			}
		} else {
			result = true;
		}
	} catch (NoClassDefFoundError e) {
	} catch (Exception e) {
		_comment("exception while creating bridge listener : ", e);
	}
	return result;
}
public final boolean createBridgeQueueManagerRemotely(String queueManagerName, String bridgeName, String bridgeQueueManagerName, boolean showErrors) {
	boolean result = false;;
	try {
		MQeMQQMgrProxyAdminMsg msg = new MQeMQQMgrProxyAdminMsg(bridgeName, bridgeQueueManagerName, false);
		msg.setAction(MQeAdminMsg.Action_Create);
		MQeAdminMsg reply = processRemoteAdminMsg(queueManagerName, msg, true);
		if (null == reply) {
			if (showErrors) {
				_error("Failed to create bridge QM " +bridgeQueueManagerName+ " on " + queueManagerName + ": no reply received");
			}
		} else if (!_isSuccess(reply)) {
			if (showErrors) {
				_error("Failed to create bridge QM " +bridgeQueueManagerName+ " on " + queueManagerName);
				_error("Reason: " + _getReason(reply));
			}
		} else {
			result = true;
		}
	} catch (NoClassDefFoundError e) {
	} catch (Exception e) {
		_comment("exception while creating bridge QM : ", e);
	}
	return result;
}
public final boolean createBridgeRemotely(String queueManagerName, String bridgeName, boolean showErrors) {
	boolean result = false;;
	try {
		MQeMQBridgeAdminMsg msg = new MQeMQBridgeAdminMsg(bridgeName, false);
		msg.setAction(MQeAdminMsg.Action_Create);
		MQeAdminMsg reply = processRemoteAdminMsg(queueManagerName, msg, true);
		if (null == reply) {
			if (showErrors) {
				_error("Failed to create bridge " +bridgeName+ " on " + queueManagerName + ": no reply received");
			}
		} else if (!_isSuccess(reply)) {
			if (showErrors) {
				_error("Failed to create bridge " +bridgeName+ " on " + queueManagerName);
				_error("Reason: " + _getReason(reply));
			}
		} else {
			result = true;
		}
	} catch (NoClassDefFoundError e) {
	} catch (Exception e) {
		_comment("exception while creating bridge: ", e);
	}
	return result;
}
protected boolean createConnectionDefinition(String remoteQueueManagerName, String route, String channel, boolean reportFailure) {
	// create a route to the remote queue manager
	boolean success = false;
	try {
		MQeConnectionAdminMsg msg = new MQeConnectionAdminMsg();
		msg.setName(remoteQueueManagerName);
		msg.create(route, null, null, channel, null);

		MQeAdminMsg reply = processLocalAdminMsg(msg, reportFailure);
		if (null != reply) {
			success = _isSuccess(reply);
		}
		if (!success && reportFailure) {
			_error("Failed to create connection: " + remoteQueueManagerName + "(" + route + ")");
			if (null == reply) {
				_error("Reason: no reply message");
			} else {
				_error("Reason: " + _getReason(reply));
			}
		}

	} catch (Exception e) {
		if (reportFailure) {
			_error("Exception creating route: " + e.getMessage());
		}
	}
	return success;
}
protected boolean createConnectionDefinitionRemotely(String fromQMName, String toQMName, String route, String channel, boolean reportFailure) {
	// create a route to the remote queue manager
	boolean success = false;
	try {
		MQeConnectionAdminMsg msg = new MQeConnectionAdminMsg();
		msg.setName(toQMName);
		msg.create(route, null, null, channel, null);

		MQeAdminMsg reply = processRemoteAdminMsg(fromQMName, msg, reportFailure);
		if (null != reply) {
			success = _isSuccess(reply);
		}
		if (!success && reportFailure) {
			_error("Failed to create connection from "+ fromQMName+" to " + toQMName + "(" + route + ")");
			if (null == reply) {
				_error("Reason: no reply message");
			} else {
				_error("Reason: " + _getReason(reply));
			}
		}

	} catch (Exception e) {
		if (reportFailure) {
			_error("Exception creating route: " + e.getMessage());
		}
	}
	return success;
}
protected boolean createHomeServerQueueRemotely(String queueManagerName, String queueQueueManagerName, String queueName, boolean showErrors) {
	boolean success = false;
	try {
		MQeHomeServerQueueAdminMsg msg = new MQeHomeServerQueueAdminMsg(queueQueueManagerName, queueName);
		MQeFields parameters = new MQeFields();
		msg.create(parameters);
		MQeAdminMsg reply = processRemoteAdminMsg(queueManagerName, msg, showErrors);
		if (null != reply) {
			success = _isSuccess(reply);
		}
		if (!success) {
			_error("Failed to create HomeServer queue: " + queueManagerName + "+" + queueName);
			_error("Reason: " + _getReason(reply));
		}

	} catch (Exception e) {
		e.printStackTrace();
		success = false;
	}
	return success;
}
protected boolean createLocalQueueRemotely(String queueManagerName, String queueName, MQeFields parameters, boolean reportErrors) {
	boolean success = false;
	try {
		if (null == parameters) {
			parameters = new MQeFields();
		}
		MQeQueueAdminMsg msg = new MQeQueueAdminMsg(queueManagerName, queueName);
		msg.create(parameters);
		MQeAdminMsg reply = processRemoteAdminMsg(queueManagerName, msg, reportErrors);
		if (null != reply) {
			success = _isSuccess(reply);
		}
		if (reportErrors && !success) {
			_error("Failed to create queue: " + queueManagerName + "+" + queueName);
			_error("Reason: " + _getReason(reply));
			checkReply(reply);
		}

	} catch (Exception e) {
		if (reportErrors) {
			_error("creating local queue remotely");
		}
			success = false;
	}
	return success;
}
protected boolean createRemoteQueueRemotely(String queueManagerName, String queueQueueManagerName, String queueName, MQeFields parameters, boolean reportErrors) {
	boolean success = false;
	try {
		if (null == parameters) {
			parameters = new MQeFields();
		}
		MQeRemoteQueueAdminMsg msg = new MQeRemoteQueueAdminMsg(queueQueueManagerName, queueName);
		msg.create(parameters);
		MQeAdminMsg reply = processRemoteAdminMsg(queueManagerName, msg, reportErrors);
		if (null != reply) {
			success = _isSuccess(reply);
		}
		if (reportErrors && !success) {
			_error("Failed to create queue: " + queueManagerName + "+" + queueName);
			_error("Reason: " + _getReason(reply));
		}

	} catch (Exception e) {
		if (reportErrors) {
			_error("creating remote queue remotely");
		}
			success = false;
	}
	return success;
}
protected boolean createStoreAndForwardQueueRemotely(String queueManagerName, String queueQueueManagerName, String queueName, boolean showErrors) {
	boolean success = false;
	try {
		MQeStoreAndForwardQueueAdminMsg msg = new MQeStoreAndForwardQueueAdminMsg(queueQueueManagerName, queueName);
		MQeFields parameters = new MQeFields();
		msg.create(parameters);
		MQeAdminMsg reply = processRemoteAdminMsg(queueManagerName, msg, showErrors);
		if (null != reply) {
			success = _isSuccess(reply);
		}
		if (!success) {
			_error("Failed to create queue: " + queueQueueManagerName + "+" + queueName + " on " + queueManagerName);
			_error("Reason: " + _getReason(reply));
		}

	} catch (Exception e) {
		if (showErrors) {
			_error("creating store and forward queue remotely", e);
		}
		success = false;
	}
	return success;
}
/**
 * Create an Asynchronous remote queue and put messages
 * to it repeatedly, until key pressed.
 */
protected boolean createSynchronousRemoteQueueRemotely(String remoteQmName, String queueQueueManagerName, String queueName, boolean showErrors) {
	MQeFields params = new MQeFields();
	boolean success = true;
	try {
		params.putByte(MQeQueueAdminMsg.Queue_Mode, MQeQueueAdminMsg.Queue_Synchronous);
		success = createRemoteQueueRemotely(remoteQmName, queueQueueManagerName, queueName, params, true);
	} catch (Exception e) {
		if (showErrors) {
			_error("creating synchronous remote queue locally", e);
		}
		success = false;
	}
	return success;
}
public final boolean deleteBridgeConnectionRemotely(String queueManagerName, String bridgeName, String bridgeQueueManagerName, String bridgeConnectionName, boolean showErrors) {
	boolean result = false;;
	try {
		MQeClientConnectionAdminMsg msg = new MQeClientConnectionAdminMsg(bridgeName, bridgeQueueManagerName, bridgeConnectionName, false);
		msg.setAction(MQeAdminMsg.Action_Delete);
		MQeAdminMsg reply = processRemoteAdminMsg(queueManagerName, msg, true);
		if (null == reply) {
			if (showErrors) {
				_error("Failed to delete bridge connection  " +bridgeConnectionName+ " on " + queueManagerName + ": no reply received");
			}
		} else if (!_isSuccess(reply)) {
			if (showErrors) {
				_error("Failed to delete bridge connection " +bridgeConnectionName+ " on " + queueManagerName);
				_error("Reason: " + _getReason(reply));
			}
		} else {
			result = true;
		}
	} catch (NoClassDefFoundError e) {
	} catch (Exception e) {
		_comment("exception while delete bridge connectionM : ", e);
	}
	return result;
}
public final boolean deleteBridgeListenerRemotely(String queueManagerName, String bridgeName, String bridgeQueueManagerName, String bridgeConnectionName, String listenerName, boolean showErrors) {
	boolean result = false;;
	try {
		MQeListenerAdminMsg msg = new MQeListenerAdminMsg(bridgeName, bridgeQueueManagerName, bridgeConnectionName, listenerName, false);
		msg.setAction(MQeAdminMsg.Action_Delete);
		MQeAdminMsg reply = processRemoteAdminMsg(queueManagerName, msg, true);
		if (null == reply) {
			if (showErrors) {
				_error("Failed to delete bridge listener " +listenerName+ " on " + queueManagerName + ": no reply received");
			}
		} else if (!_isSuccess(reply)) {
			if (showErrors) {
				_error("Failed to delete bridge listener " +listenerName+ " on " + queueManagerName);
				_error("Reason: " + _getReason(reply));
			}
		} else {
			result = true;
		}
	} catch (NoClassDefFoundError e) {
	} catch (Exception e) {
		_comment("exception while delete bridge listener : ", e);
	}
	return result;
}
public final boolean deleteBridgeQueueManagerRemotely(String queueManagerName, String bridgeName, String bridgeQueueManagerName, boolean showErrors) {
	boolean result = false;;
	try {
		MQeMQQMgrProxyAdminMsg msg = new MQeMQQMgrProxyAdminMsg(bridgeName, bridgeQueueManagerName, false);
		msg.setAction(MQeAdminMsg.Action_Delete);
		MQeAdminMsg reply = processRemoteAdminMsg(queueManagerName, msg, true);
		if (null == reply) {
			if (showErrors) {
				_error("Failed to delete bridge QM " +bridgeQueueManagerName+ " on " + queueManagerName + ": no reply received");
			}
		} else if (!_isSuccess(reply)) {
			if (showErrors) {
				_error("Failed to delete bridge QM " +bridgeQueueManagerName+ " on " + queueManagerName);
				_error("Reason: " + _getReason(reply));
			}
		} else {
			result = true;
		}
	} catch (NoClassDefFoundError e) {
	} catch (Exception e) {
		_comment("exception while delete bridge QM : ", e);
	}
	return result;
}
public final boolean deleteBridgeRemotely(String queueManagerName, String bridgeName, boolean showErrors) {
	boolean result = false;;
	try {
		MQeMQBridgeAdminMsg msg = new MQeMQBridgeAdminMsg(bridgeName, false);
		msg.setAction(MQeAdminMsg.Action_Delete);
		MQeAdminMsg reply = processRemoteAdminMsg(queueManagerName, msg, true);
		if (null == reply) {
			if (showErrors) {
				_error("Failed to create delete " +bridgeName+ " on " + queueManagerName + ": no reply received");
			}
		} else if (!_isSuccess(reply)) {
			if (showErrors) {
				_error("Failed to create delete " +bridgeName+ " on " + queueManagerName);
				_error("Reason: " + _getReason(reply));
			}
		} else {
			result = true;
		}
	} catch (NoClassDefFoundError e) {
	} catch (Exception e) {
		_comment("exception while delete bridge: ", e);
	}
	return result;
}
protected boolean deleteConnectionDefinitionRemotely(String queueManagerName, String remoteQueueManagerName, boolean reportFailure) {
	// create a route to the remote queue manager
	boolean success = false;
	try {
		MQeConnectionAdminMsg msg = new MQeConnectionAdminMsg();
		msg.setName(remoteQueueManagerName);
		msg.delete(new MQeFields());
		MQeAdminMsg reply = processRemoteAdminMsg(queueManagerName, msg, reportFailure);
		if (null != reply) {
			success = _isSuccess(reply);
		}
		if (!success && reportFailure) {
			_error("Failed to delete connection: " + remoteQueueManagerName);
			_error("Reason: " + _getReason(reply));
		}

	} catch (Exception e) {
		if (reportFailure) {
			System.out.println("Exception creating route: " + e.getMessage());
		}
	}
	return success;
}
protected boolean deleteQueueRemotely(String queueManagerName, String queueQueueManagerName, String queueName, boolean showErrors) {
	boolean success = false;
	try {
		MQeQueueAdminMsg msg = new MQeQueueAdminMsg(queueQueueManagerName, queueName);
		MQeFields parameters = new MQeFields();
		msg.delete(parameters);
		MQeAdminMsg reply = processRemoteAdminMsg(queueManagerName, msg, showErrors);
		if (null != reply) {
			success = _isSuccess(reply);
		}
		if (showErrors && !success) {
			_error("Failed to delete queue: " + queueQueueManagerName + "+" + queueName);
			_error("Reason: " + _getReason(reply));
		}

	} catch (Exception e) {
		if (showErrors) {
			_error("deleting local queue locally", e);
		}
			success = false;
	}
	return success;
}
protected final String getLocalQueueManagerName() {
	return localQueueManagerName;
}
public final MQeFields inquireAllBridgeConnectionRemotely(String queueManagerName, String bridgeName, String bridgeQMName, String bridgeConnectionName, boolean showErrors) {
	MQeFields result = null;
	try {
		MQeClientConnectionAdminMsg msg = new MQeClientConnectionAdminMsg(bridgeName, bridgeQMName, bridgeConnectionName, false);
		msg.setAction(MQeAdminMsg.Action_InquireAll);
		MQeAdminMsg reply = processRemoteAdminMsg(queueManagerName, msg, true);
		if (null == reply) {
			if (showErrors) {
				_error("Failed to inquireAll bridgeConnection " +bridgeConnectionName+ " on " + queueManagerName + ": no reply received");
			}
		} else if (!_isSuccess(reply)) {
			if (showErrors) {
				_error("Failed to inquireAll bridgeConnection " +bridgeConnectionName+ " on " + queueManagerName);
				_error("Reason: " + _getReason(reply));
			}
		} else {
			result = reply.getOutputFields();
		}
	} catch (NoClassDefFoundError e) {
	} catch (Exception e) {
		_comment("exception while inquireAll bridgeConnection: ", e);
	}
	return result;
}
public final MQeFields inquireAllBridgeQueueManagerRemotely(String queueManagerName, String bridgeName, String bridgeQMName, boolean showErrors) {
	MQeFields result = null;
	try {
		MQeMQQMgrProxyAdminMsg msg = new MQeMQQMgrProxyAdminMsg(bridgeName, bridgeQMName, false);
		msg.setAction(MQeAdminMsg.Action_InquireAll);
		MQeAdminMsg reply = processRemoteAdminMsg(queueManagerName, msg, true);
		if (null == reply) {
			if (showErrors) {
				_error("Failed to inquireAll bridgeQM " +bridgeQMName+ " on " + queueManagerName + ": no reply received");
			}
		} else if (!_isSuccess(reply)) {
			if (showErrors) {
				_error("Failed to inquireAll bridgeQM " +bridgeQMName+ " on " + queueManagerName);
				_error("Reason: " + _getReason(reply));
			}
		} else {
			result = reply.getOutputFields();
		}
	} catch (NoClassDefFoundError e) {
	} catch (Exception e) {
		_comment("exception while inquireAll bridge: ", e);
	}
	return result;
}
public final MQeFields inquireAllBridgeRemotely(String queueManagerName, String bridgeName, boolean showErrors) {
	MQeFields result = null;
	try {
		MQeMQBridgeAdminMsg msg = new MQeMQBridgeAdminMsg(bridgeName, false);
		msg.setAction(MQeAdminMsg.Action_InquireAll);
		MQeAdminMsg reply = processRemoteAdminMsg(queueManagerName, msg, true);
		if (null == reply) {
			if (showErrors) {
				_error("Failed to inquireAll bridge " +bridgeName+ " on " + queueManagerName + ": no reply received");
			}
		} else if (!_isSuccess(reply)) {
			if (showErrors) {
				_error("Failed to inquireAll bridge " +bridgeName+ " on " + queueManagerName);
				_error("Reason: " + _getReason(reply));
			}
		} else {
			result = reply.getOutputFields();
		}
	} catch (NoClassDefFoundError e) {
	} catch (Exception e) {
		_comment("exception while inquireAll bridge: ", e);
	}
	return result;
}
protected MQeConnectionAdminMsg inquireAllConnectionDefinitionRemotely(String queueManagerName, String destQMName, boolean showErrors) {
	MQeConnectionAdminMsg reply = null;
	boolean success = false;
	try {
		MQeConnectionAdminMsg msg = new MQeConnectionAdminMsg(destQMName);
		MQeFields parameters = new MQeFields();
		msg.inquireAll(null /*parameters*/);
		reply = (MQeConnectionAdminMsg) processRemoteAdminMsg(queueManagerName, msg, showErrors);
		if (null != reply) {
			success = _isSuccess(reply);
		}
		if (!success) {
			_error("Failed to inquire bridge: " + destQMName);
			_error("Reason: " + _getReason(reply));
			reply = null;
		}

	} catch (Exception e) {
		e.printStackTrace();
		reply = null;
		success = false;
	}
	return reply;
}
/**
 * perform an enquire all on a remote queue on the local machine
 * return the reply
 */
protected MQeQueueManagerAdminMsg inquireAllQueueManagerRemotely(String queueManagerName, boolean showErrors) {
	MQeQueueManagerAdminMsg reply = null;
	try {
		MQeQueueManagerAdminMsg msg = new MQeQueueManagerAdminMsg(queueManagerName);
		MQeFields parameters = new MQeFields();
		msg.inquireAll(parameters);
		reply = (MQeQueueManagerAdminMsg)processRemoteAdminMsg(queueManagerName, msg, showErrors);
		boolean success = false;
		if (null != reply) {
			success = _isSuccess(reply);
		}
		if (!success && showErrors) {
			_error("Failed to inquire queueManager: " + queueManagerName);
			if (null == reply) {
				_error("Reason: no reply");
			} else {
				_error("Reason: " + _getReason(reply));
			}
		}

	} catch (Exception e) {
		if (showErrors) {
			e.printStackTrace();
		}
	}
	return reply;
}
protected MQeQueueAdminMsg inquireAllQueueRemotely(String queueManagerName, String queueQueueManagerName, String queueName, boolean showErrors) {
	MQeQueueAdminMsg reply = null;
	try {
		MQeQueueAdminMsg msg = new MQeQueueAdminMsg(queueQueueManagerName, queueName);
		MQeFields parameters = new MQeFields();
		msg.inquireAll(parameters);
		reply = (MQeQueueAdminMsg)processRemoteAdminMsg(queueManagerName, msg, showErrors);
		if (null == reply) {
			_error("Failed to inquire queue: " + queueQueueManagerName + "+" + queueName + " on " + queueManagerName
								+ " no reply received");
		} else if (!_isSuccess(reply)) {
			_error("Failed to inquire queue: " + queueQueueManagerName + "+" + queueName + " on " + queueManagerName);
			_error("Reason: " + _getReason(reply));
		}

	} catch (Exception e) {
		e.printStackTrace();
	}
	return reply;
}
protected final MQeAdminMsg processLocalAdminMsg(MQeAdminMsg msg, boolean showErrors) {
	MQeAdminMsg reply = null;
	try {
		msg.setTargetQMgr(getLocalQueueManagerName());
		String match = addLocalAdminReplyRequestTo(msg);
		boolean success = putLocalAdminMessage(msg, showErrors);
		if (success) {
			reply = waitForLocalAdminReply(match, 10000, showErrors);
		}
	} catch (Exception e) {
		_error("Handling local admin request", e);
	}
	return reply;
}
protected final MQeAdminMsg processRemoteAdminMsg(String queueManagerName, MQeAdminMsg msg, boolean showErrors) {
	MQeAdminMsg reply = null;
	try {
		msg.setTargetQMgr(queueManagerName);
		String match = addRemoteAdminReplyRequestTo(queueManagerName, msg);
		boolean success = putMessage(queueManagerName, MQe.Admin_Queue_Name, msg, showErrors);
		if (success) {
			reply = waitForRemoteAdminReply(queueManagerName, match, 20000, showErrors);
		}
	} catch (Exception e) {
		_error("Handling local admin request", e);
	}
	return reply;
}
protected final boolean putLocalAdminMessage(MQeMsgObject msg, boolean showErrors) {
	return putMessage(getLocalQueueManagerName(),
		MQe.Admin_Queue_Name,
		msg, showErrors);
}
protected final boolean putMessage(String queueManagerName, String queueName, MQeMsgObject msg, boolean showErrors) {
	return putMessage(queueManagerName, queueName, msg, null, 0, showErrors);
}
protected final void putRemoteAdminMessage(String queueManagerName, MQeMsgObject msg, boolean showErrors) {
	putMessage(queueManagerName,
		MQe.Admin_Queue_Name,
		msg, null, 0L, showErrors);
}
protected boolean removeAliasFromConnectionDefinitionRemotely(String targetQueueManagerName, String remoteQueueManagerName, String alias, boolean reportFailure) {
	// create a route to the remote queue manager
	boolean success = false;
	try {
		MQeConnectionAdminMsg msg = new MQeConnectionAdminMsg();
		msg.setName(remoteQueueManagerName);
		msg.removeAlias(alias);

		MQeAdminMsg reply = processRemoteAdminMsg(targetQueueManagerName, msg, reportFailure);
		if (null != reply) {
			success = _isSuccess(reply);
		}
		if (!success && reportFailure) {
			_error("Failed to remove connection alias: " + alias + " from " + remoteQueueManagerName);
			_error("Reason: " + _getReason(reply));
		}

	} catch (Exception e) {
		if (reportFailure) {
			_error("Exception removing alias: " + e.getMessage());
		}
	}
	return success;
}
public final boolean removeQueueAliasRemotely(String queueManagerName, String queueQueueManagerName, String queueName, String alias, boolean showErrors) {
	boolean success = false;
	
	try {
		MQeQueueAdminMsg msg = new MQeQueueAdminMsg(queueQueueManagerName, queueName);
		msg.removeAlias(alias);
		MQeAdminMsg reply = processRemoteAdminMsg(queueManagerName, msg, showErrors);
		if (null != reply) {
			success = _isSuccess(reply);
		}
		if (showErrors && !success) {
			_error("Failed to remove alias " + alias + " from queue: " + queueName +"@"+ queueManagerName);
			_error("Reason: " + _getReason(reply));
			checkReply(reply);
		}

	} catch (Exception e) {
		if (showErrors) {
			_error("Failed to remove alias " + alias + " from queue: " + queueName +"@"+ queueManagerName, e);
		}
			success = false;
	}
	
	return success;
}
protected boolean removeQueueManagerFromStoreAndForwardQueueRemotely(String destQMName, String queueQueueManagerName, String queueName, String addQmName, boolean showErrors) {
	boolean success = false;
	try {
		MQeStoreAndForwardQueueAdminMsg msg = new MQeStoreAndForwardQueueAdminMsg(queueQueueManagerName, queueName);
		MQeFields parameters = new MQeFields();
		msg.removeQueueManager(addQmName);
		MQeAdminMsg reply = processRemoteAdminMsg(destQMName, msg, showErrors);
		if (null != reply) {
			success = _isSuccess(reply);
		}
		if (!success) {
			_error("Failed to add queue manager " + addQmName + " to S&F queue " + queueQueueManagerName + "+" + queueName);
			_error("Reason: " + _getReason(reply));
		}

	} catch (Exception e) {
		e.printStackTrace();
		success = false;
	}
	return success;
}
public boolean start(Hashtable args) {
	localQueueManagerName = (String)args.get(ARG_QUEUEMANAGER_NAME);
	return super.start(args);
}
public final boolean updateBridgeConnectionRemotely(String queueManagerName, String bridgeName, String bridgeQMName, String bridgeConnectionName, MQeFields params, boolean showErrors) {
	boolean result = false;;
	try {
		MQeClientConnectionAdminMsg msg = new MQeClientConnectionAdminMsg(bridgeName, bridgeQMName, bridgeConnectionName, false);
		msg.update(params);
		MQeAdminMsg reply = processRemoteAdminMsg(queueManagerName, msg, true);
		if (null == reply) {
			if (showErrors) {
				_error("Failed to update bridgeConnection " +bridgeQMName+ " on " + queueManagerName + ": no reply received");
			}
		} else if (!_isSuccess(reply)) {
			if (showErrors) {
				_error("Failed to update bridgeConnection " +bridgeQMName+ " on " + queueManagerName);
				_error("Reason: " + _getReason(reply));
			}
		} else {
			result = true;
		}
	} catch (NoClassDefFoundError e) {
	} catch (Exception e) {
		_comment("exception while updating bridgeConnection: ", e);
	}
	return result;
}
public final boolean updateBridgeQueueManagerRemotely(String queueManagerName, String bridgeName,  String bridgeQMName,MQeFields params, boolean showErrors) {
	boolean result = false;;
	try {
		MQeMQQMgrProxyAdminMsg msg = new MQeMQQMgrProxyAdminMsg(bridgeName, bridgeQMName, false);
		msg.update(params);
		MQeAdminMsg reply = processRemoteAdminMsg(queueManagerName, msg, true);
		if (null == reply) {
			if (showErrors) {
				_error("Failed to update bridgeQM " +bridgeQMName+ " on " + queueManagerName + ": no reply received");
			}
		} else if (!_isSuccess(reply)) {
			if (showErrors) {
				_error("Failed to update bridgeQM " +bridgeQMName+ " on " + queueManagerName);
				_error("Reason: " + _getReason(reply));
			}
		} else {
			result = true;
		}
	} catch (NoClassDefFoundError e) {
	} catch (Exception e) {
		_comment("exception while updating bridgeQM: ", e);
	}
	return result;
}
public final boolean updateBridgeRemotely(String queueManagerName, String bridgeName, MQeFields params, boolean showErrors) {
	boolean result = false;;
	try {
		MQeMQBridgeAdminMsg msg = new MQeMQBridgeAdminMsg(bridgeName, false);
		msg.update(params);
		MQeAdminMsg reply = processRemoteAdminMsg(queueManagerName, msg, true);
		if (null == reply) {
			if (showErrors) {
				_error("Failed to update bridge " +bridgeName+ " on " + queueManagerName + ": no reply received");
			}
		} else if (!_isSuccess(reply)) {
			if (showErrors) {
				_error("Failed to update bridge " +bridgeName+ " on " + queueManagerName);
				_error("Reason: " + _getReason(reply));
			}
		} else {
			result = true;
		}
	} catch (NoClassDefFoundError e) {
	} catch (Exception e) {
		_comment("exception while updating bridge: ", e);
	}
	return result;
}
protected boolean updateConnectionRemotely(String queueManagerName, String connectionName, MQeFields params, boolean showErrors) {
	boolean success = false;
	try {
		if (null == params) {
			params = new MQeFields();
		}
		MQeConnectionAdminMsg msg = new MQeConnectionAdminMsg(connectionName);
		msg.update(params);
		MQeAdminMsg reply = processRemoteAdminMsg(queueManagerName, msg, showErrors);
		if (null != reply) {
			success = checkReply(reply);
		}
		if (!success) {
			_error("Failed to update connection: " + connectionName);
		}

	} catch (Exception e) {
		e.printStackTrace();
		success = false;
	}
	return success;
}
protected boolean updateQueueManagerRemotely(String queueManagerName, MQeFields params, boolean showErrors) {
	boolean success = false;
	try {
		if (null == params) {
			params = new MQeFields();
		}
		MQeQueueManagerAdminMsg msg = new MQeQueueManagerAdminMsg(queueManagerName);
		msg.update(params);
		MQeAdminMsg reply = processRemoteAdminMsg(queueManagerName, msg, showErrors);
		if (null != reply) {
			success = checkReply(reply);
		}
		if (!success) {
			_error("Failed to update queueManager: " + queueManagerName);
		}

	} catch (Exception e) {
		e.printStackTrace();
		success = false;
	}
	return success;
}
protected boolean updateQueueRemotely(String queueManagerName, String queueQueueManagerName, String queueName, MQeFields params, boolean showErrors) {
	boolean success = false;
	try {
		if (null == params) {
			params = new MQeFields();
		}
		MQeQueueAdminMsg msg = new MQeQueueAdminMsg(queueQueueManagerName, queueName);
		msg.update(params);
		MQeAdminMsg reply = processRemoteAdminMsg(queueManagerName, msg, showErrors);
		if (null != reply) {
			success = checkReply(reply);
		}
		if (!success) {
			_error("Failed to update queue: " + queueName+"@"+queueQueueManagerName+" on "+queueManagerName);
		}

	} catch (Exception e) {
		e.printStackTrace();
		success = false;
	}
	return success;
}
/**
   * Wait for message - waits for a mesaage to
   * arrive on the reply queue
   * return message, or null if timed out
   */
protected final MQeAdminMsg waitForAdminReply(String queueManagerName, String match, int totalWaitTime, boolean showErrors) {
	MQeFields filter = new MQeFields();
	try {
		filter.putArrayOfByte(MQe.Msg_CorrelID, match.getBytes());
	} catch (Exception e) {
	}

	MQeAdminMsg msg = null;
	boolean error = false;
	while (!error && null == msg && totalWaitTime > 0) {
		try {
			msg = (MQeAdminMsg)getMessage(queueManagerName, MQe.Admin_Reply_Queue_Name,
									filter, null, 0L);
		} catch (MQeException e) {
			if (e.code() == MQe.Except_Q_NoMatchingMsg) {
				System.out.println("admin wait");
				try {
					Thread.sleep(1000);
				} catch (InterruptedException ee) {
				}
				totalWaitTime -= 1000;
			} else {
				error = true;
				_error("waiting for admin reply",e);
			}
		} catch (Exception e) {
			error = true;
			_error("waiting for admin reply",e);
		}
		 
	}
	
	return msg;
}
/**
   * Wait for message - waits for a mesaage to
   * arrive on the reply queue
   * return message, or null if timed out
   */
protected final MQeAdminMsg waitForLocalAdminReply(String match, int totalWaitTime, boolean showErrors) {
	return (MQeAdminMsg)waitForAdminReply(getLocalQueueManagerName(), match, totalWaitTime, showErrors);
}
/**
   * Wait for message - waits for a mesaage to
   * arrive on the reply queue
   * return message, or null if timed out
   */
protected final MQeAdminMsg waitForRemoteAdminReply(String remoteQueueManagerName, String match, int totalWaitTime, boolean showErrors) {
	MQeFields filter = new MQeFields();
	try {
		filter.putArrayOfByte(MQe.Msg_CorrelID, match.getBytes());
	} catch (Exception e) {
	}

	return (MQeAdminMsg)waitForAdminReply(remoteQueueManagerName, match, totalWaitTime, showErrors);
}
}
