package designer.gui;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import designer.model.*;
import java.awt.*;
import java.util.Vector;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
public class ListenerSelectDialogue extends glyph.dialogues.RendererDialogue implements ItemListener {
	private final Choice 	hostChoice 		= new Choice();
	private final Choice 	qmChoice	 	= new Choice();
	private final Choice 	listenerChoice 	= new Choice();
	private Vector listeners;
	private Listener selectedListener = null;
/**
 * DetailsDialogue constructor comment.
 * @param parent java.awt.Frame
 * @param title java.lang.String
 */
public ListenerSelectDialogue(Dialog parent, String title, Vector listeners) {
	super(parent, title);
	_setup(listeners);
	pack();
}
/**
 * DetailsDialogue constructor comment.
 * @param parent java.awt.Frame
 * @param title java.lang.String
 */
public ListenerSelectDialogue(Frame parent, String title, Vector listeners) {
	super(parent, title);
	_setup(listeners);
	pack();
}
private void _selectListener(String hostName, String qmName, String listenerName) {
	selectedListener = null;
    for (int i = 0;null == selectedListener && i < listeners.size(); i++) {
        Listener listener = (Listener) listeners.elementAt(i);
        QueueManager queueManager = listener.getQueueManager();
        Host host = queueManager.getHost();
        if (host.getName().equals(hostName) && queueManager.getName().equals(qmName) && listener.getName().equals(listenerName)) {
            selectedListener = listener;
        }
    }
}
private final void _setup(Vector listeners) {
	this.listeners = listeners;
	Panel panel = new Panel(new GridLayout(3,2));

		// 	set up the visual bits
	hostChoice.addItemListener(this);
	hostChoice.setEnabled(true);
	
	qmChoice.addItemListener(this);
	qmChoice.removeAll();

	listenerChoice.addItemListener(this);
	listenerChoice.removeAll();

	_setupHostList();

	// arrange the visual bits
	panel.add(new Label("Host:"));
	panel.add(hostChoice);

	panel.add(new Label("QueueManager:"));
	panel.add(qmChoice);
	
	panel.add(new Label("Listener:"));
	panel.add(listenerChoice);

	add(BorderLayout.CENTER, panel);
	add(BorderLayout.SOUTH, createOkCancelPanel());
}
private void _setupHostList() {
	Vector added = new Vector();
	hostChoice.removeAll();
	for (int i = 0; i < listeners.size(); i++){
		Listener listener = (Listener)listeners.elementAt(i);
		QueueManager queueManager = listener.getQueueManager();
		Host host = queueManager.getHost();
		String hostName = host.getName();
		if (!added.contains(hostName)) {
			hostChoice.add(hostName);
			added.addElement(hostName);
		}
	}
	if (added.size() > 0) {
		String str = (String)added.elementAt(0);
		hostChoice.select(str);
		_setupQueueManagerList(str);
	}
}
private void _setupListenerList(String hostName, String qmName) {
    Vector added = new Vector();
    listenerChoice.removeAll();
    listenerChoice.setEnabled(true);
    
    for (int i = 0; i < listeners.size(); i++) {
        Listener listener = (Listener) listeners.elementAt(i);
        QueueManager queueManager = listener.getQueueManager();
        Host host = queueManager.getHost();
        if (host.getName().equals(hostName) && queueManager.getName().equals(qmName)) {
            String listenerName = listener.getName();
            if (!added.contains(listenerName)) {
                listenerChoice.add(listenerName);
                added.addElement(listenerName);
            }
        }
    }
    if (listenerChoice.getItemCount() > 0) {
	    listenerChoice.select(0);
    }
}
private void _setupQueueManagerList(String hostName) {
    Vector added = new Vector();
    qmChoice.removeAll();
    qmChoice.setEnabled(true);
    listenerChoice.setEnabled(false);
    
    for (int i = 0; i < listeners.size(); i++) {
        Listener listener = (Listener) listeners.elementAt(i);
        QueueManager queueManager = listener.getQueueManager();
        Host host = queueManager.getHost();
        if (host.getName().equals(hostName)) {
            String qmName = queueManager.getName();
            if (!added.contains(qmName)) {
                qmChoice.add(qmName);
                added.addElement(qmName);
            }
        }
    }
	if (added.size() > 0) {
		String str = (String)added.elementAt(0);
		qmChoice.select(str);
		_setupListenerList(hostName, str);
    }
}
	/**
	 * Invoked when an item has been selected or deselected.
	 * The code written for this method performs the operations
	 * that need to occur when an item is selected (or deselected).
	 */
public void itemStateChanged(ItemEvent e) {
	if (e.getSource() == hostChoice) {
		String hostName = hostChoice.getSelectedItem();
    	if (null != hostName) {
			_setupQueueManagerList(hostName);
    	}
	} else if (e.getSource() == qmChoice) {
		String hostName = hostChoice.getSelectedItem();
		String qmName = qmChoice.getSelectedItem();
    	if (null != hostName && null != qmName) {
			_setupListenerList(hostName, qmName);
    	}
	} else if (e.getSource() == listenerChoice) {
		String hostName = hostChoice.getSelectedItem();
		String qmName = qmChoice.getSelectedItem();
    	String listenerName = listenerChoice.getSelectedItem();
    	if (null != hostName && null != qmName && null != listenerName) {
			_selectListener(hostName, qmName, listenerName);
    	}
	}
}
public final Listener selectedListener() {
	if (null == selectedListener) {
		// some gui paths do not explicitly select....
		String hostName = hostChoice.getSelectedItem();
		String qmName = qmChoice.getSelectedItem();
    	String listenerName = listenerChoice.getSelectedItem();
    	if (null != hostName && null != qmName && null != listenerName) {
			_selectListener(hostName, qmName, listenerName);
    	}
	}
	return selectedListener;
}
}
