package designer.model;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import java.awt.PopupMenu;
import java.awt.Menu;
import java.awt.MenuItem;

import java.awt.event.ActionEvent;
/**
 * Insert the type's description here.
 * Creation date: (14/09/2001 16:42:00)
 * @author: 
 */
public class AbstractRemoteQueue extends Queue {
    public static final String ATTRIBUTE_REMQ_TRANSPORTER = "Transporter";
    public static final String ATTRIBUTE_REMQ_TRANSPORTER_XOR = "Transporter XOR";
    public static final String ATTRIBUTE_REMQ_CLOSE_IDLE = "Close When Idle";
    private static final String MI_SHOWDEFAULT = "Show Default Connections";
    private static final String MI_HIDEDEFAULT = "Hide Default Connections";
    protected boolean showDefault = true;

    private DefaultRelationship usesConnection = null;
/**
 * AbstractRemoteQueue constructor comment.
 * @param owner snoop.model.QueueManager
 */
public AbstractRemoteQueue(QueueManager owner) {
	super(owner);

	createAttribute(ATTRIBUTE_REMQ_TRANSPORTER, "DefaultTransporter", true);
	createAttribute(ATTRIBUTE_REMQ_TRANSPORTER_XOR, "false", true);
	createAttribute(ATTRIBUTE_REMQ_CLOSE_IDLE, "false", true);
}
public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();
    if (MI_SHOWDEFAULT.equals(cmd)) {
        showDefault = true;
        declareChanged();
    } else if (MI_HIDEDEFAULT.equals(cmd)) {
        showDefault = false;
        declareChanged();
    } else {
        super.actionPerformed(e);
    }
}
/**
 * delete this node if not already deleted
 */
public void delete() {
	if (null != usesConnection) {
		usesConnection.detach();
	}
	super.delete();
}
/**
 * A remote queue will resolve to a connection (hopefully) and then use that
 * connection to determine the end point
 */
public final Connection getConnection() {
	return getQueueManager().getConnection(getQueueManagerName());
}
/**
 * Get the string representing the receiver in a gui
 */
public String getDisplayName() {
	return getName()+"@"+getQueueManagerName();
}
public PopupMenu getMenu() {
	PopupMenu popup = super.getMenu();
	MenuItem menuItem;
	if (showDefault) {
		menuItem = new MenuItem(MI_HIDEDEFAULT);
	} else {
		menuItem = new MenuItem(MI_SHOWDEFAULT);
	}
	menuItem.addActionListener(this);
	popup.add(menuItem);
	return popup;
}
/**
 * Method caled when a change occurs that may need a 
 * recalculation - when a model object is deleted or
 * similar.  Sublasses should overide as appropriate
 */
protected void notifyChange() {
    // if there is a relationship, check to see if it is still valid
    // and if not then cut it free
    if (null != usesConnection && !usesConnection.isAttached()) {
            usesConnection = null;
    }
    if (!isToBeDeleted()) {
        Connection conn = conn = getQueueManager().getConnection(getQueueManagerName());
        if (null != conn) {
            if (null == usesConnection) {
                usesConnection = new DefaultRelationship(this, conn, "connects using");
            }
        }
    }

    if (null != usesConnection) {
        usesConnection.setVisible(showDefault && ShowDefaultRelationships);
    }
}
}
