package designer.model;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import java.util.Vector;
import java.util.Enumeration;

import java.awt.PopupMenu;
import java.awt.Menu;
import java.awt.MenuItem;
import java.awt.Color;

/**
 * Insert the type's description here.
 * Creation date: (12/09/2001 07:13:24)
 * @author: 
 */
public class BridgeConnection extends AbstractBridgeObject {
	private static final String MI_CREATE_LISTENER = "Add Listener";
	private static final String MI_CREATE_BRIDGEQUEUE = "Add Bridge Queue";
	private static final Color BRIDGE_CONNECTION_COLOR = new Color(255,160,255);
    private static final String MI_SHOWDEFAULT = "Show Default Connection";
    private static final String MI_HIDEDEFAULT = "Hide Default Connection";
    private boolean showDefault = true;

	private BridgeQueueManager bridgeQueueManager;
	private Vector bridgeListeners = new Vector();
    private DefaultRelationship defaultRelationship = null;
/**
 * Insert the method's description here.
 * Creation date: (12/09/2001 07:33:47)
 * @param bridgeQueueManager snoop.model.BridgeQueueManager
 */
public BridgeConnection(BridgeQueueManager bridgeQueueManager) {
	super();
	this.bridgeQueueManager = bridgeQueueManager;
	createAttribute("Startup Rules","",false);
	createAttribute("Port","",false);
	createAttribute("Adapter","",false);
	createAttribute("MQ User ID","",false);
	createAttribute("MQ Password","",false);
	createAttribute("Receive Exit","",false);
	createAttribute("Send Exit","",false);
	createAttribute("Security Exit","",false);
	createAttribute("CCSID","0",false);
	createAttribute("Sync Queue Name","",false);
	createAttribute("Sync Queue Purger Rule","",false);
	createAttribute("Sync Queue Purge Interval","0",false);
	createAttribute("Max Connection Idle Time","0",false);
}
public void actionPerformed(java.awt.event.ActionEvent e) {
	String cmd = e.getActionCommand();
	if (MI_SHOWDEFAULT.equals(cmd)) {
        showDefault = true;
        declareChanged();
    } else if (MI_HIDEDEFAULT.equals(cmd)) {
        showDefault = false;
        declareChanged();
    } else if (MI_CREATE_LISTENER.equals(cmd)) {
		String data[][] = new String[][] {{"Listener Name", "Listener"},};
		if (getView().getStringData("Add Listener", data)) {
			newBridgeListener(data[0][1]);
		}
	} else if (MI_CREATE_BRIDGEQUEUE.equals(cmd)) {
		String data[][] = new String[][] {{"Bridge Queue Name", "BridgeQueue"},};
		if (getView().getStringData("Add Bridge Queue", data)) {
			getQueueManager().createBridgeQueue(
					getBridgeQueueManagerName(),
					data[0][1],
					getBridgeName(),
					getName());
		}
	} else {
		super.actionPerformed(e);
	}
}
/**
 * delete this node if not already deleted
 */
public void delete() {
    for (int i = 0; i < bridgeListeners.size(); i++){
    	((AbstractBridgeObject)bridgeListeners.elementAt(i)).delete();
    }
    super.delete();
}
protected final AdministrationInterface getAdministrator() {
	return bridgeQueueManager.getAdministrator();
}
public final BridgeListener getBridgeListener(String name) {
	BridgeListener found = null;
	for (int i = 0; null == found &&  i < bridgeListeners.size(); i++){
		BridgeListener listener = (BridgeListener)bridgeListeners.elementAt(i);
		if (listener.getName().equals(name)) {
			found = listener;
		}
	}
	return found;
}
public final Enumeration getBridgeListeners() {
	return bridgeListeners.elements();
}
public final String getBridgeName() {
	return bridgeQueueManager.getBridgeName();
}
public final BridgeQueueManager getBridgeQueueManager() {
	return bridgeQueueManager;
}
public final String getBridgeQueueManagerName() {
	return bridgeQueueManager.getName();
}
public String getDescription() {
	return getShortDescription() + " on " + bridgeQueueManager.getDescription();
}
/**
 * Search among the recievers children for a bridge listener pulling
 * messages from the specified transmit queue.
 * return null if not found
 */
BridgeListener getListenerFor(MQXmitQueue txQueue) {
    BridgeListener bridgeListener = null;
    for (int i = 0; null == bridgeListener && i < bridgeListeners.size(); i++) {
        bridgeListener =((BridgeListener) bridgeListeners.elementAt(i)).getListenerFor(txQueue);
    }
    return bridgeListener;
}
public PopupMenu getMenu() {
    PopupMenu popup = super.getMenu();
    MenuItem menuItem;

    menuItem = new MenuItem(MI_CREATE_LISTENER);
    menuItem.addActionListener(this);
    popup.add(menuItem);

    menuItem = new MenuItem(MI_CREATE_BRIDGEQUEUE);
    menuItem.addActionListener(this);
    popup.add(menuItem);

    if (showDefault) {
        menuItem = new MenuItem(MI_HIDEDEFAULT);
    } else {
        menuItem = new MenuItem(MI_SHOWDEFAULT);
    }
    menuItem.addActionListener(this);
    popup.add(menuItem);

    return popup;
}
/**
 * reply the model to which the receiver belongs
 */
protected final MQeModel getModel() {
	return bridgeQueueManager.getModel();
}
public Model getParent() {
	return bridgeQueueManager;
}
public final QueueManager getQueueManager() {
	return bridgeQueueManager.getQueueManager();
}
public String getType() {
	return "Bridge Connection";
}
public boolean isBridgeConnection() {
	return true;
}
public final BridgeListener newBridgeListener(String name) {
	BridgeListener bridgeListener = getBridgeListener(name);
	if (null == bridgeListener) {
		bridgeListener = new BridgeListener(this);
		bridgeListeners.addElement(bridgeListener);
		bridgeListener.setName(name);
		addNode(bridgeListener);
	}
	return bridgeListener;
}
/**
 * Method caled when a change occurs that may need a 
 * recalculation - when a model object is deleted or
 * similar.  Sublasses should overide as appropriate
 */
protected void notifyChange() {
	for (int i = 0; i < bridgeListeners.size(); i++){
		((BridgeListener)bridgeListeners.elementAt(i)).notifyChange();
	}
	if (null != defaultRelationship) {
		// if there is a relationship, check to see if it is still valid
		// and if not then cut it free
		if (!defaultRelationship.isAttached()) {
			defaultRelationship = null;
		}
	}
	if (null == defaultRelationship && !isToBeDeleted()) {
		MQQueueManager mqqm = bridgeQueueManager.getMQQueueManager();
		if (null != mqqm) {
			MQChannel channel = mqqm.getChannel(getName());
			if (null != channel) {
				defaultRelationship = new DefaultRelationship(this, channel, "connects using");
			}
		}
	}
    if (null != defaultRelationship) {
	    defaultRelationship.setVisible(showDefault && ShowDefaultRelationships);
    }
}
/**
 * Called to request that the receiver removes itself
 * from model
 */
public final void performDelete() {
	bridgeQueueManager.removeConnection(this);
	declareDeleted();
	delete();
}
public final void removeListener(BridgeListener listener) {
	bridgeListeners.removeElement(listener);
	declareChanged();
}
}
