package designer.model;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import java.awt.PopupMenu;
import java.awt.Menu;
import java.awt.MenuItem;

import java.awt.event.ActionEvent;
import java.awt.Color;
/**
 * Insert the type's description here.
 * Creation date: (12/09/2001 07:14:09)
 * @author: 
 */
public class BridgeListener extends AbstractBridgeObject {
	private static final Color BRIDGE_LISTENER_COLOR = new Color(255,190,255);
    private static final String MI_SHOWDEFAULT = "Show Default Connection";
    private static final String MI_HIDEDEFAULT = "Hide Default Connection";
    private boolean showDefault = true;
	private BridgeConnection bridgeConnection;
    private DefaultRelationship defaultRelationship = null;

    BridgePullMessageRoute messageRoute;
/**
 * Insert the method's description here.
 * Creation date: (12/09/2001 07:35:58)
 * @param bridgeConnection snoop.model.BridgeConnection
 */
public BridgeListener(BridgeConnection bridgeConnection) {
	super();
	this.bridgeConnection = bridgeConnection;
	createAttribute("Startup Rules","",false);
	createAttribute("Dead Letter Queue","",false);
	createAttribute("Listener State Store Adapter","",false);
	createAttribute("Undelivered Message Rule","",false);
	createAttribute("Transformer","",false);
}
/**
 * Resolve the put message route.  This is done by adding steps to the route
 * until the destination is reached (or found to be unreachable)
 *
 * This method takes the put from a queue manager to a queue (local or remote) on
 * the queue manager.  If the queue is a remote queue then the resolution is
 * continued by the queue.
 */
final AbstractStep _resolveMessageRoute(AbstractStep previousStep, String qmName, String qName) {
    AbstractStep step = previousStep;
    if (!isAttached()) {
	    step = super._resolveMessageRoute(previousStep, qmName, qName);
    }
    getQueueManager()._resolveMessageRoute(step, qmName, qName);
	return step;
}
public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();
    if (MI_SHOWDEFAULT.equals(cmd)) {
        showDefault = true;
        declareChanged();
    } else if (MI_HIDEDEFAULT.equals(cmd)) {
        showDefault = false;
        declareChanged();
    } else {
        super.actionPerformed(e);
    }
}
protected final AdministrationInterface getAdministrator() {
	return bridgeConnection.getAdministrator();
}
public final String getBridgeConnectionName() {
	return bridgeConnection.getName();
}
public final String getBridgeName() {
	return bridgeConnection.getBridgeName();
}
public final String getBridgeQueueManagerName() {
	return bridgeConnection.getBridgeQueueManagerName();
}
public String getDescription() {
	return getShortDescription() + " on " + bridgeConnection.getDescription();
}
/**
 * Search among the recievers children for a bridge listener pulling
 * messages from the specified transmit queue.
 * return null if not found
 */
BridgeListener getListenerFor(MQXmitQueue txQueue) {
    BridgeListener bridgeListener = null;
    if (txQueue.getName().equals(getName())) {
	    bridgeListener = this;
    }
    return bridgeListener;
}
public PopupMenu getMenu() {
	PopupMenu popup = super.getMenu();
	MenuItem menuItem;
	if (showDefault) {
		menuItem = new MenuItem(MI_HIDEDEFAULT);
	} else {
		menuItem = new MenuItem(MI_SHOWDEFAULT);
	}
	menuItem.addActionListener(this);
	popup.add(menuItem);
	return popup;
}
/**
 * reply the model to which the receiver belongs
 */
protected final MQeModel getModel() {
	return bridgeConnection.getModel();
}
public Model getParent() {
	return bridgeConnection;
}
public final QueueManager getQueueManager() {
	return bridgeConnection.getQueueManager();
}
public String getType() {
	return "Bridge Listener";
}
public boolean isBridgeListener() {
	return true;
}
/**
 * Method caled when a change occurs that may need a 
 * recalculation - when a model object is deleted or
 * similar.  Sublasses should overide as appropriate
 */
protected void notifyChange() {
	if (null != defaultRelationship) {
		// if there is a relationship, check to see if it is still valid
		// and if not then cut it free
		if (!defaultRelationship.isAttached()) {
			defaultRelationship = null;
		}
	}
    if (null != messageRoute && !messageRoute.isValid()) {
        getQueueManager().removeMessageRoute(messageRoute);
	    messageRoute.delete();
        messageRoute = null;
    }
	if (!isToBeDeleted()) {
		MQQueueManager mqqm = bridgeConnection.getBridgeQueueManager().getMQQueueManager();
		if (null != mqqm) {
			MQQueue xmitQ = mqqm.getQueue(getName());
			if (null != xmitQ && xmitQ.isMQTransmissionQueue()) {
				if (null == defaultRelationship) {
					defaultRelationship = new DefaultRelationship(this, xmitQ, "pulls from");
				}
				if (null == messageRoute) {
					messageRoute = new BridgePullMessageRoute(this, xmitQ);
					getQueueManager().addMessageRoute(messageRoute);
				}
				if (null != messageRoute) {
					messageRoute.setNames(((MQXmitQueue)xmitQ).getNameList());
				}
			}
		}
	}
    if (null != defaultRelationship) {
	    defaultRelationship.setVisible(showDefault && ShowDefaultRelationships);
    }
}
/**
 * Called to request that the receiver removes itself
 * from model
 */
public final void performDelete() {
	bridgeConnection.removeListener(this);
	declareDeleted();
	delete();
}
public boolean performRefresh() {
	boolean success = false;
	info("Refresh Bridge: " + getName());
	//MQeQueueAdminMsg msg = owner.performEnquireQueue(queueManagerName, queueName);
	//if (null != msg) {
	//	success = refresh(msg);
	//}
	return success;
}
}
