package designer.model;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import java.awt.PopupMenu;
import java.awt.Menu;
import java.awt.MenuItem;

import java.awt.event.ActionEvent;
/**
 * Insert the type's description here.
 * Creation date: (14/09/2001 16:40:04)
 * @author: 
 */
public class BridgeQueue extends Queue {
	private static final String ATTRIBUTE_BRIDGE_NAME = "BridgeName";
	private static final String ATTRIBUTE_CONNECTION_NAME = "ConnectionName";
	private static final String ATTRIBUTE_BRIDGEHOST_NAME = "BridgeHostName";
    private static final String MI_SHOWDEFAULT = "Show Default Connection";
    private static final String MI_HIDEDEFAULT = "Hide Default Connection";
    private boolean showDefault = true;

	private BridgeMessageRoute messageRoute;
    private DefaultRelationship defaultRelationship = null;
/**
 * BridgeQueue constructor comment.
 * @param owner snoop.model.QueueManager
 */
public BridgeQueue(QueueManager owner) {
	super(owner);
	createAttribute("Maximum Idle Time", "5", true);
	createAttribute("Transformer","",false);
	createAttribute(ATTRIBUTE_BRIDGE_NAME, "", true);
	createAttribute(ATTRIBUTE_BRIDGEHOST_NAME, "", true);
	createAttribute(ATTRIBUTE_CONNECTION_NAME, "", true);
}
/**
 * Resolve the put message route.  This is done by adding steps to the route
 * until the destination is reached (or found to be unreachable)
 *
 * This method takes the put from a queue manager to a queue (local or remote) on
 * the queue manager.  If the queue is a remote queue then the resolution is
 * continued by the queue.
 */
final AbstractStep _resolveMessageRoute(AbstractStep previousStep, String qmName, String qName) {
    AbstractStep step = super._resolveMessageRoute(previousStep, qmName, qName);
    MQQueue mqq = resolveMQQueue();
    if (null != mqq) {
	    mqq._resolveMessageRoute(step, qmName, qName);
    }
    return step;
}
public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();
    if (MI_SHOWDEFAULT.equals(cmd)) {
        showDefault = true;
        declareChanged();
    } else if (MI_HIDEDEFAULT.equals(cmd)) {
        showDefault = false;
        declareChanged();
    } else {
        super.actionPerformed(e);
    }
}
/**
 * delete this node if not already deleted
 */
public void delete() {
	if (null != defaultRelationship) {
		defaultRelationship.detach();
	}
    if (null != messageRoute) {
        getQueueManager().removeMessageRoute(messageRoute);
	    messageRoute.delete();
        messageRoute = null;
    }
	super.delete();
}
public final String getBridgeName() {
	return getAttribute(ATTRIBUTE_BRIDGE_NAME).getValue();
}
public final String getConnectionName() {
	return getAttribute(ATTRIBUTE_CONNECTION_NAME).getValue();
}
/**
 * Get the string representing the receiver in a gui
 */
public String getDisplayName() {
	return getName()+"@"+getQueueManagerName();
}
public PopupMenu getMenu() {
	PopupMenu popup = super.getMenu();
	MenuItem menuItem;
	if (showDefault) {
		menuItem = new MenuItem(MI_HIDEDEFAULT);
	} else {
		menuItem = new MenuItem(MI_SHOWDEFAULT);
	}
	menuItem.addActionListener(this);
	popup.add(menuItem);
	return popup;
}
public String getType() {
	return "Bridge Queue";
}
public boolean isBridgeQueue() {
	return true;
}
/**
 * Method caled when a change occurs that may need a 
 * recalculation - when a model object is deleted or
 * similar.  Sublasses should overide as appropriate
 */
protected void notifyChange() {
    if (null != defaultRelationship) {
        // if there is a relationship, check to see if it is still valid
        // and if not then cut it free
        if (!defaultRelationship.isAttached()) {
            defaultRelationship = null;
        }
    }
    if (null != messageRoute && !messageRoute.isValid()) {
        getQueueManager().removeMessageRoute(messageRoute);
	    messageRoute.delete();
        messageRoute = null;
    }
    if (null == defaultRelationship) {
        MQQueue mqq = resolveMQQueue();
        if (null != mqq) {
            defaultRelationship = new DefaultRelationship(this, mqq, "resolves to");
            messageRoute = new BridgeMessageRoute(this, mqq);
            getQueueManager().addMessageRoute(messageRoute);
        }
    }
    if (null != defaultRelationship) {
        defaultRelationship.setVisible(showDefault && ShowDefaultRelationships);
    }
}
/**
 * Resolve dstination bridge queue
 */
MQQueue resolveMQQueue() {
    MQQueue queue = null;
    Bridge bridge = getQueueManager().getBridge(getBridgeName());
    if (null != bridge) {
        BridgeQueueManager bqm = bridge.getBridgeQueueManager(getQueueQueueManagerName());
        if (null != bqm) {
            MQQueueManager mqqm = bqm.getMQQueueManager();
            if (null != mqqm) {
                queue = mqqm.getQueue(getQueueName());
            }
        }
    }
    return queue;
}
public final void setBridgeHostName(String name) {
	setAttribute(ATTRIBUTE_BRIDGEHOST_NAME, name);
}
public final void setBridgeName(String name) {
	setAttribute(ATTRIBUTE_BRIDGE_NAME, name);
}
public final void setConnectionName(String name) {
	setAttribute(ATTRIBUTE_CONNECTION_NAME, name);
}
}
