package designer.model;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import java.util.Vector;
import java.util.Enumeration;

import java.awt.Menu;
import java.awt.MenuShortcut;
import java.awt.MenuItem;
import java.awt.MenuBar;
import java.awt.Color;
import java.awt.Frame;
import java.awt.CheckboxMenuItem;
import java.awt.FileDialog;
import java.awt.PrintJob;

import java.awt.event.WindowListener; 
import java.awt.event.WindowEvent; 
import java.awt.event.ActionListener; 
import java.awt.event.ActionEvent; 
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

import java.awt.event.KeyEvent;		// for virtual key definitions

import glyph.*;

/**
 * This class provides the root visible entity for the snoop gui.
 * GUI elements should NOT know about MQe.
 * Creation date: (20/08/2001 12:07:14)
 * @author: 
 */
public class Designer extends Frame implements WindowListener, ActionListener, ItemListener {
  public final static String copyright =
		   "Licensed Materials - Property of IBM\n" +
		   "Product number: 5765-E63\n" +
		   "Copyright IBM Corp. 1999  All Rights Reserved. " +
		   "US Government Users Restriced Rights - use, duplication or" +
		   "disclosure restriced by GSA ADP Schedule Contract with IBM Corp.";
	private static final String MI_ADD_HOST = "Add Host";
	private static final String MI_EXPERT = "Expert Mode";
	
	private static final String MI_SETFONT = "Set Font"; 
	private static final String MI_SHOWHELP = "Show Help";
	private static final String MI_ABOUT = "About";
	private static final String MI_EXIT = "Exit";
	private static final String MI_PRINT = "Print";
	private static final String MI_SAVEDIAGRAM = "Save Diagram";
	private static final String MI_SAVE_BITMAP = "Bitmap (BMP)";
	private static final String MI_SAVE_METAFILE = "Metafile (WMP)";
	private static final String MI_SAVEMODEL = "Save Model (XML)";
	private static final String MI_LOADMODEL = "Load Model (XML)";
	private static final String MI_NEWMODEL = "New Model";

	private static final String MI_RESOLVEQM = "Show QMs in Message Routes"; 
	private static final String MI_RESOLVEHOST = "Show Hosts in Message Routes";
	
	private static final String MI_SHOWHEADINGS = "Show Object Types";
	private static final String MI_VIEWDEFAULTRELATIONSHIPS = "Show Default Relationships";
	private static final String MI_DUMPXML = "Dump XML";
	private static final String MI_SHOWCHILDREN = "Show Children";
	

	private View view;
	private String currentBitmapDirectory = null;
	private MQeModel mqeModel;
	private CheckboxMenuItem menuItemUseExpertMode;
	private CheckboxMenuItem menuItemIncludeHostInResolution;
	private CheckboxMenuItem menuItemIncludeQMInResolution;
	private CheckboxMenuItem menuItemViewDefaultRelationships;
	private String currentModelFileName = "(new)";
/**
 * Designer constructor comment.
 * @param title java.lang.String
 */
public Designer(String title) {
    super();
    addWindowListener(this);
    view = new View();
    view.setFrame(this);
    add(view);
	this.mqeModel = new MQeModel();
	mqeModel.setView(view);
    setMenuBar(_createDesignerMenu());
    setTitle("(new)");
}
private final void __performAddHost() {
	String[][] data = new String[][] {
		{"IP Address", "localhost",}};
	if (view.getStringData("Add Host", data)) {
		mqeModel.getOrCreateHost(data[0][1]);
	}
}
private final MenuBar _createDesignerMenu() {
	MenuBar menuBar = new MenuBar();
	////////// File Menu //////////////////////////////////
	Menu menu = new Menu("File");
	Menu saveMenu = new Menu("Save Diagram");
	MenuItem menuItem = new MenuItem(MI_SAVE_BITMAP);
	//menuItem.setShortcut(new MenuShortcut(KeyEvent.VK_S, true));
	menuItem.setActionCommand(menuItem.getLabel());
	menuItem.addActionListener(this);
	saveMenu.add(menuItem);

	menuItem = new MenuItem(MI_SAVE_METAFILE);
	//menuItem.setShortcut(new MenuShortcut(KeyEvent.VK_S, true));
	menuItem.setActionCommand(menuItem.getLabel());
	menuItem.addActionListener(this);
	saveMenu.add(menuItem);
	menu.add(saveMenu);
	
	menuItem = new MenuItem(MI_PRINT);
	menuItem.setShortcut(new MenuShortcut(KeyEvent.VK_P));
	menuItem.setActionCommand(menuItem.getLabel());
	menuItem.addActionListener(this);
	menu.add(menuItem);
	
	menuItem = new MenuItem(MI_SAVEMODEL);
	menuItem.setShortcut(new MenuShortcut(KeyEvent.VK_S));
	menuItem.setActionCommand(menuItem.getLabel());
	menuItem.addActionListener(this);
	menu.add(menuItem);
	
	menuItem = new MenuItem(MI_LOADMODEL);
	menuItem.setShortcut(new MenuShortcut(KeyEvent.VK_L));
	menuItem.setActionCommand(menuItem.getLabel());
	menuItem.addActionListener(this);
	menu.add(menuItem);
	
	menuItem = new MenuItem(MI_NEWMODEL);
	menuItem.setShortcut(new MenuShortcut(KeyEvent.VK_N));
	menuItem.setActionCommand(menuItem.getLabel());
	menuItem.addActionListener(this);
	menu.add(menuItem);
	
	menu.addSeparator();
	menuItem = new MenuItem(MI_EXIT);
	menuItem.setShortcut(new MenuShortcut(KeyEvent.VK_Q));
	menuItem.setActionCommand(menuItem.getLabel());
	menuItem.addActionListener(this);
	menu.add(menuItem);
	menuBar.add(menu);
	////////////////// Edit Menu ///////////////////////
	menu = new Menu("Edit");
	menuItem = new MenuItem(MI_ADD_HOST);
	menuItem.setShortcut(new MenuShortcut(KeyEvent.VK_H));
	menuItem.setActionCommand(menuItem.getLabel());
	menuItem.addActionListener(this);
	menu.add(menuItem);
	
	menuItemUseExpertMode = new CheckboxMenuItem(MI_EXPERT);
	menuItemUseExpertMode.addItemListener(this);
	menuItemUseExpertMode.setState(Model.UseExpertMode);
	menu.add(menuItemUseExpertMode);

	menuBar.add(menu);

	//////////View Menu //////////////////////////////////////
	menu = new Menu("View");
	menuItemViewDefaultRelationships = new CheckboxMenuItem(MI_VIEWDEFAULTRELATIONSHIPS);
	menuItemViewDefaultRelationships.setShortcut(new MenuShortcut(KeyEvent.VK_D));
	menuItemViewDefaultRelationships.setState(true);
	menuItemViewDefaultRelationships.addItemListener(this);
	menu.add(menuItemViewDefaultRelationships);

	menuItemIncludeHostInResolution = new CheckboxMenuItem(MI_RESOLVEHOST);
	menuItemIncludeHostInResolution.setShortcut(new MenuShortcut(KeyEvent.VK_R));
	menuItemIncludeHostInResolution.addItemListener(this);
	menuItemIncludeHostInResolution.setState(Model.IncludeHostResolution);
	menu.add(menuItemIncludeHostInResolution);

	menuItemIncludeQMInResolution = new CheckboxMenuItem(MI_RESOLVEQM);
	menuItemIncludeQMInResolution.addItemListener(this);
	menuItemIncludeQMInResolution.setState(true);
	menu.add(menuItemIncludeQMInResolution);

	menuItem = new MenuItem(MI_SHOWCHILDREN);
	menuItem.setShortcut(new MenuShortcut(KeyEvent.VK_C));
	menuItem.setActionCommand(menuItem.getLabel());
	menuItem.addActionListener(this);
	menu.add(menuItem);
	
	menuItem = new MenuItem(MI_SETFONT);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuBar.add(menu);
	
	///////////////////// Help Menu ///////////////////////////////
	menu = new Menu("Help");
	menuItem = new MenuItem(MI_SHOWHELP);
	menuItem.setShortcut(new MenuShortcut(KeyEvent.VK_H, true));
	menuItem.setActionCommand(menuItem.getLabel());
	menuItem.addActionListener(this);
	menu.add(menuItem);
	menuItem = new MenuItem(MI_ABOUT);
	menuItem.setShortcut(new MenuShortcut(KeyEvent.VK_A));
	menuItem.setActionCommand(menuItem.getLabel());
	menuItem.addActionListener(this);
	menu.add(menuItem);
	menuBar.add(menu);

	return menuBar;	
}
public void actionPerformed(ActionEvent e) {
    String command = e.getActionCommand();
    if (MI_SETFONT.equals(command)) {
        new glyph.dialogues.ChooseFont(this, view).show();
    } else if (MI_ADD_HOST.equals(command)) {
        __performAddHost();
    } else if (MI_ABOUT.equals(command)) {
            String about = //"Author: Phill van Leersum of IBM UK Laboratories" +
    "\nWebsphere MQ Series Designer is supplied 'as is' with no official support." + "\nVersion: 1.012\n\n" + copyright;

        glyph.dialogues.Splash dlg = new glyph.dialogues.Splash(this, "About Websphere MQ Series Designer", about);
        dlg.setBounds(getBounds());
        dlg.show();
    } else if (MI_SAVEMODEL.equals(command)) {
        saveModel();
    } else if (MI_LOADMODEL.equals(command)) {
        loadModel();
    } else if (MI_NEWMODEL.equals(command)) {
        mqeModel.removeAllHosts();
        //view.clear();
        view.repaint();
        currentModelFileName = "(new)";
        setTitle(currentModelFileName);
    } else if (MI_SAVE_BITMAP.equals(command)) {
        saveBitmap();
    } else if (MI_SAVE_METAFILE.equals(command)) {
        saveMetafile();
    } else if (MI_PRINT.equals(command)) {
        printDiagram();
    } else if (MI_EXIT.equals(command)) {
        System.exit(0);
    } else if (MI_SHOWCHILDREN.equals(command)) {
        mqeModel.setChildrenVisible();
    } else if (MI_SHOWHELP.equals(command)) {
        view.showHelp(null, true);
    }
}
/**
 * Invoked when an item has been selected or deselected.
 * The code written for this method performs the operations
 * that need to occur when an item is selected (or deselected).
 */
public void itemStateChanged(ItemEvent e) {
    if (e.getSource() == menuItemViewDefaultRelationships) {
        Model.ShowDefaultRelationships = menuItemViewDefaultRelationships.getState();
        mqeModel.declareChanged();
        repaint();
    } else if (e.getSource() == menuItemIncludeHostInResolution) {
        Model.IncludeHostResolution = menuItemIncludeHostInResolution.getState();
        mqeModel.declareChanged();
        repaint();
    } else if (e.getSource() == menuItemIncludeQMInResolution) {
        Model.IncludeQueueManagerResolution = menuItemIncludeQMInResolution.getState();
        mqeModel.declareChanged();
        repaint();
    } else if (e.getSource() == menuItemUseExpertMode) {
	    Model.UseExpertMode = menuItemUseExpertMode.getState();
	}
}
private final void loadModel() {
	FileDialog fd = new FileDialog(this, "Load Model", FileDialog.LOAD);
	java.io.File curdir = new java.io.File(".");
	//fd.setDirectory(curdir.getAbsolutePath());
	fd.setFile("*.xml");
	fd.show();
	String filename = fd.getFile();
	String directoryName = fd.getDirectory();
	if (null != filename && null != directoryName) {
		currentModelFileName = directoryName+filename;
		XML.ParseFileInto(currentModelFileName, mqeModel);
		mqeModel.declareChanged();
		setTitle(currentModelFileName);
	}
}
/**
 * Insert the method's description here.
 * Creation date: (20/08/2001 12:07:36)
 * @param args java.lang.String[]
 */
public static void main(String[] args) {
	Designer frame = new Designer("Websphere MQ Everyplace Designer");
	frame.setBounds(20,20,620,620);
	frame.show();
}
protected final void printDiagram() {
	PrintJob pj = getToolkit().getPrintJob(this, "Print Diagram", null);
	if (null != pj) {
		view.print(pj.getGraphics());
		pj.end();
	}
}
protected final void saveBitmap() {
	FileDialog fd = new FileDialog(this, "Save Bitmap", FileDialog.SAVE);
	if (null != currentBitmapDirectory) {
		fd.setDirectory(currentBitmapDirectory+"*.bmp");
	}
	fd.setFile("*.bmp");
	fd.show();
	String filename = fd.getFile();
	if (!filename.endsWith(".bmp")) {
		filename = filename + ".bmp";
	}
	String directoryName = fd.getDirectory();
	if (null != filename && null != directoryName) {
		currentBitmapDirectory = directoryName/*+filename*/;
		view.saveBitmap(currentBitmapDirectory+filename);
		
	}
}
protected final void saveMetafile() {
	FileDialog fd = new FileDialog(this, "Save Metafile", FileDialog.SAVE);
	if (null != currentBitmapDirectory) {
		fd.setDirectory(currentBitmapDirectory+"*.wmf");
	}
	fd.setFile("*.wmf");
	fd.show();
	String filename = fd.getFile();
	if (!filename.endsWith(".wmf")) {
		filename = filename + ".wmf";
	}
	String directoryName = fd.getDirectory();
	if (null != filename && null != directoryName) {
		currentBitmapDirectory = directoryName/*+filename*/;
		view.saveMetafile(currentBitmapDirectory+filename);
		
	}
}
private final void saveModel() {
	FileDialog fd = new FileDialog(this, "Save Model", FileDialog.SAVE);
	//if (null != currentBitmapDirectory) {
	//	fd.setDirectory(currentBitmapDirectory);
	//}
	fd.setFile("*.xml");
	fd.show();
	String filename = fd.getFile();
	if (!filename.endsWith(".xml")) {
		filename = filename + ".xml";
	}
	String directoryName = fd.getDirectory();
	if (null != filename && null != directoryName) {
		String xml = mqeModel.getXML();
		try {
			java.io.OutputStreamWriter out = new java.io.OutputStreamWriter(
				new java.io.BufferedOutputStream(
					new java.io.FileOutputStream(directoryName+filename)));
			out.write(xml);
			out.close();
			currentModelFileName = directoryName+filename;
			setTitle(currentModelFileName);
		} catch (java.io.IOException ioe) {
			System.out.println(ioe.toString());
		}
	}
}
	/**
	 * Sets the title for this frame to the specified string.
	 * @param    title    the title to be displayed in the frame's border
	 * @param title the title to be displayed in the frame's border.
	 *              A <code>null</code> value
	 *              is treated as an empty string, "".
	 * @see      java.awt.Frame#getTitle
	 */
	public void setTitle(String title) {
		String me = "WMQeDesigner: ";
		if (-1 == title.indexOf(me)) {
			if (null == title || 0 == title.length()) {
				title = "(new)";
			}
			title = me + title;
		}
		super.setTitle(title);
	}
	public void update(java.awt.Graphics g) {
		paint(g);
	}
	/**
	 * Invoked when the window is set to be the user's
	 * active window, which means the window (or one of its
	 * subcomponents) will receive keyboard events.
	 */
public void windowActivated(WindowEvent e) {}
	/**
	 * Invoked when a window has been closed as the result
	 * of calling dispose on the window.
	 */
public void windowClosed(WindowEvent e) {}
	/**
	 * Invoked when the user attempts to close the window
	 * from the window's system menu.  If the program does not 
	 * explicitly hide or dispose the window while processing 
	 * this event, the window close operation will be cancelled.
	 */
public void windowClosing(WindowEvent e) {
	dispose();
	System.exit(0);
}
	/**
	 * Invoked when a window is no longer the user's active
	 * window, which means that keyboard events will no longer
	 * be delivered to the window or its subcomponents.
	 */
public void windowDeactivated(WindowEvent e) {}
	/**
	 * Invoked when a window is changed from a minimized
	 * to a normal state.
	 */
public void windowDeiconified(WindowEvent e) {}
	/**
	 * Invoked when a window is changed from a normal to a
	 * minimized state. For many platforms, a minimized window 
	 * is displayed as the icon specified in the window's 
	 * iconImage property.
	 * @see Frame#setIconImage
	 */
public void windowIconified(WindowEvent e) {}
	/**
	 * Invoked the first time a window is made visible.
	 */
public void windowOpened(WindowEvent e) {

}
}
