package designer.model;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/
public final class Listener extends AttributedModel {
	public static final String ATTRIBUTE_LISTENER_ADAPTER = "Adapter";
	public static final String ATTRIBUTE_LISTENER_PORT = "Port";

	private QueueManager owningQueueManager;
/**
 * Listener constructor comment.
 */
public Listener(QueueManager owningQueueManager, String name, String port, String adapter) {
	super();
	this.owningQueueManager = owningQueueManager;
	setName(name);
	createAttribute(ATTRIBUTE_LISTENER_ADAPTER, adapter, false);
	createAttribute(ATTRIBUTE_LISTENER_PORT, port, false);
}
/**
 * Resolve the put message route.  This is done by adding steps to the route
 * until the destination is reached (or unreachable
 *
 * Most subclasses will be quite content with the creation method used here 
 * - the step representing the receiver will be returned for the benefit of the
 * caller.  If the receiver wishes to propagate the resolution it should
 * provide an alternate implementation of this method.
 */
AbstractStep _resolveMessageRoute(AbstractStep previousStep, String qmName, String qName) {
    AbstractStep step = previousStep;
    if (!isAttached()) {
	    step = super._resolveMessageRoute(previousStep, qmName, qName);
    }
    owningQueueManager._resolveMessageRoute(step, qmName, qName);
    return step;
}
public final String getAdapter() {
	return getAttribute(ATTRIBUTE_LISTENER_ADAPTER).getValue();
}
/**
 * getAdministrator method comment.
 */
protected final AdministrationInterface getAdministrator() {
	return owningQueueManager.getAdministrator();
}
public String getDescription() {
	return getShortDescription()
		+ " defined on " + owningQueueManager.getDescription();
}
public String getExplanation() {
	StringBuffer buf = new StringBuffer();
	buf.append(getShortDescription());
	buf.append(" defines the means by which ");
	buf.append(getQueueManager().getShortDescription());
	buf.append(" accepts incoming communications from another queue manager ");
	buf.append(". This listener is using the protocol represented by ");
	buf.append(getAdapter());
	buf.append(".");
	return buf.toString();
}
/**
 * reply the model to which the receiver belongs
 */
protected final MQeModel getModel() {
	return owningQueueManager.getModel();
}
public final String getPort() {
	return getAttribute(ATTRIBUTE_LISTENER_PORT).getValue();
}
public QueueManager getQueueManager() {
	return owningQueueManager;
}
public String getType() {
	return "Listener";
}
/**
 * Called to request that the receiver removes itself
 * from model
 */
public final void performDelete() {
	owningQueueManager.removeListener(this);
	declareDeleted();
}
}
