package designer.model;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import java.util.Vector;
import java.util.Enumeration;
import java.awt.PopupMenu;
import java.awt.Menu;
import java.awt.MenuItem;

public class MQQueueManager extends MQResource {
	private static final String MI_CREATE_LOCAL_QUEUE = "Local Queue";
	private static final String MI_CREATE_REMOTE_QUEUE = "Remote Queue";
	private static final String MI_CREATE_TRANSMISSION_QUEUE = "Transmission Queue";
	private static final String MI_CREATE_CHANNEL = "Create Channel";

	private static final String MI_SHOW_MSG_ROUTE = "Show Message Route";
	private static final String MI_HIDE_MSG_ROUTE = "Hide Message Route";
	

	private Host host;
	private final Category queueCategory = new Category(this, "Queues");
	private final Category channelCategory = new Category(this, "Channels");
	private String targetQueueName = "TargetQueue";
	private String targetQueueManagerName = "TargetQM";
	private MessageResolution messageResolution = new MessageResolution(this);
/**
 * Resolve the put message route.  This is done by adding steps to the route
 * until the destination is reached (or found to be unreachable)
 *
 * This method takes the put from a queue manager to a queue (local or remote) on
 * the queue manager.  If the queue is a remote queue then the resolution is
 * continued by the queue.
 */
final AbstractStep __continueMessageRoute(AbstractStep step, String qmName, String qName) {
    // the queueName-queueManagerName pair will resolve to a logical queue
    // which then indicates the real queue
    MQQueue queue = getQueue(qmName, qName);
    if (null == queue) {
	    // look for queue manager alias
	    queue = getQueue(qmName, qmName);
    }
    if (null != queue) {
        queue._resolveMessageRoute(step, qmName, qName);
    }
    return step;
}
/**
 * Resolve the put message route.  This is done by adding steps to the route
 * until the destination is reached (or found to be unreachable)
 *
 * This method takes the put from a queue manager to a queue (local or remote) on
 * the queue manager.  If the queue is a remote queue then the resolution is
 * continued by the queue.
 */
final AbstractStep _resolveMessageRoute(AbstractStep previousStep, String qmName, String qName) {
    AbstractStep step = previousStep;
    if (!isAttached()) {
	    step = super._resolveMessageRoute(previousStep, qmName, qName);
    }
    __continueMessageRoute(step, qmName, qName);
    return step;
}
public void actionPerformed(java.awt.event.ActionEvent e) {
    String cmd = e.getActionCommand();
    if (MI_SHOW_MSG_ROUTE.equals(cmd)) {
        String data[][] = new String[][] { { "Queue Name", targetQueueName }, {
                "QueueManager Name", targetQueueManagerName }, };
        if (getView().getStringData("Put Message", data)) {
            if (null != messageResolution) {
                messageResolution.setVisible(false);
                messageResolution = null;
                declareChanged();
            }
            targetQueueName = data[0][1];
            targetQueueManagerName = data[1][1];
            messageResolution = new MessageResolution(this);
            __continueMessageRoute(messageResolution, targetQueueManagerName, targetQueueName);
            messageResolution.calculateLines();
            declareChanged();
        }
    } else if (MI_HIDE_MSG_ROUTE.equals(cmd)) {
        if (null != messageResolution) {
            messageResolution.setVisible(false);
            messageResolution = null;
            declareChanged();
        }
    } else if (MI_DELETE.equals(cmd)) {
        delete();
        host.removeMQQueueManager(this);
    } else if (MI_CREATE_LOCAL_QUEUE.equals(cmd)) {
        String data[][] = new String[][] { { "Queue Name", "MQ_LocalQueue" }, };
        if (getView().getStringData("Create MQ Local Queue", data)) {
            createLocalQueue(data[0][1]);
        }
    } else if (MI_CREATE_REMOTE_QUEUE.equals(cmd)) {
        String data[][] = new String[][] { { "QueueManager Name", "MQ_RemoteQM" }, {
                "Queue Name", "MQ_RemoteQueue" }, {
                "Tx Queue", "TXQUEUE" }
        };
        if (getView().getStringData("Create MQ Remote Queue", data)) {
            //if (queueManager.queueExists(getName(), data[0][1])) {
            // throw error here
            //} else {
            MQRemoteQueue rq = createRemoteQueue(data[1][1]);
            rq.setRemoteQueueManagerName(data[0][1]);
            rq.setTxQueueName(data[2][1]);
            //}
        }
    } else if (MI_CREATE_TRANSMISSION_QUEUE.equals(cmd)) {
        String data[][] = new String[][] { { "QueueManager Name", "MQ_RemoteQM" }, {
                "Queue Name", "MQ_TransmissionQueue" }, };
        if (getView().getStringData("Create MQ Transmission Queue", data)) {
            //if (queueManager.queueExists(getName(), data[0][1])) {
            // throw error here
            //} else {
            createTransmissionQueue(data[1][1]);
            //}
        }
    } else if (MI_CREATE_CHANNEL.equals(cmd)) {
        String data[][] = new String[][] { { "Channel Name", "MQ_Channel" }, };
        if (getView().getStringData("Create MQ Channel", data)) {
            createChannel(data[0][1]);
        }
    } else {
        super.actionPerformed(e);
    }
}
public final MQChannel createChannel(String channelName) {
	MQChannel channel = getChannel(channelName);
	if (null == channel) {
		channel = new MQChannel(this);
		channel.setName(channelName);
		channelCategory.addNode(channel);
		//queue.commit();
	}
	return channel;
}
public final MQLocalQueue createLocalQueue(String queueName) {
	MQLocalQueue localQueue = null;
	MQQueue queue = getQueue(queueName);
	if (null == queue) {
		localQueue = new MQLocalQueue(this);
		localQueue.setName(queueName);
		queueCategory.addNode(localQueue);
		//queue.commit();
	} else if (queue.isMQLocalQueue()) {
		localQueue = (MQLocalQueue)queue;
	}
		
	return localQueue;
}
public final MQRemoteQueue createRemoteQueue(String queueName) {
	MQRemoteQueue remoteQueue = null;
	MQQueue queue = getQueue(queueName);
	if (null == queue) {
		remoteQueue = new MQRemoteQueue(this);
		queueCategory.addNode(remoteQueue);
		remoteQueue.setName(queueName);
		// default remote name to same as name
		remoteQueue.setRemoteQueueName(queueName);
		
		//queue.commit();
	} else if (queue.isMQRemoteQueue()) {
		remoteQueue = (MQRemoteQueue)queue;
	}
		
	return remoteQueue;
}
public final MQXmitQueue createTransmissionQueue(String queueName) {
	MQXmitQueue txQueue = null;
	MQQueue queue = getQueue(queueName);
	if (null == queue) {
		txQueue = new MQXmitQueue(this);
		txQueue.setName(queueName);
		queueCategory.addNode(txQueue);;
		//queue.commit();
	} else if (queue.isMQRemoteQueue()) {
		txQueue = (MQXmitQueue)queue;
	}
		
	return txQueue;
}
/**
 * delete this node if not already deleted
 */
public void delete() {
    queueCategory.delete();
    channelCategory.delete();
    super.delete();
}
public final MQChannel getChannel(String channelName) {
	return (MQChannel)channelCategory.getChild(channelName);
}
public final Enumeration getChannels() {
	return channelCategory._getChildren().elements();
}
public String getDescription() {
	return getShortDescription() +
		" running on " + host.getDescription();
}
/**
 * return  a textual description of the receiver
 */
public String getExplanation() {
	StringBuffer buf = new StringBuffer();
	buf.append(getShortDescription());
	buf.append(" provides an entry point to the MQ system for applications running on ");
	buf.append(host.getShortDescription());
	buf.append(".");
	return buf.toString();
}
public PopupMenu getMenu() {
	PopupMenu popup = super.getMenu();
	MenuItem menuItem;
	
	Menu addMenu = new Menu("Create Queue");
	
	menuItem = new MenuItem(MI_CREATE_LOCAL_QUEUE);
	menuItem.addActionListener(this);
	addMenu.add(menuItem);
	menuItem = new MenuItem(MI_CREATE_REMOTE_QUEUE);
	menuItem.addActionListener(this);
	addMenu.add(menuItem);
	menuItem = new MenuItem(MI_CREATE_TRANSMISSION_QUEUE);
	menuItem.addActionListener(this);
	addMenu.add(menuItem);
	popup.add(addMenu);

	menuItem = new MenuItem(MI_CREATE_CHANNEL);
	menuItem.addActionListener(this);
	popup.add(menuItem);

	menuItem = new MenuItem(MI_SHOW_MSG_ROUTE);
	menuItem.addActionListener(this);
	popup.add(menuItem);
	if (null != messageResolution) {
		if (messageResolution.hasVisibleLines()) {
			menuItem = new MenuItem(MI_HIDE_MSG_ROUTE);
			menuItem.addActionListener(this);
			popup.add(menuItem);
		}
	}	

	return popup;
}
/**
 * reply the model to which the receiver belongs
 */
protected final MQeModel getModel() {
	return host.getModel();
}
public Model getParent() {
	return host;
}
public final MQQueue getQueue(String queueName) {
	return (MQQueue)queueCategory.getChild(queueName);
}
public final MQQueue getQueue(String queueManagerName, String queueName) {
	MQQueue found = null;
	Enumeration enum = getQueues();
	while (null == found && enum.hasMoreElements()) {
		MQQueue queue =(MQQueue)enum.nextElement();
		if (queue.matches(queueManagerName, queueName)) {
			found = queue;
		}
	}
	return found;
}
public final Enumeration getQueues() {
	return queueCategory._getChildren().elements();
}
public final Vector getQueuesVector() {
	return queueCategory._getChildren();
}
public String getType() {
	return "MQ Queue Manager";
}
public final boolean isMQQueueManager() {
	return true;
}
/**
 * Method caled when a change occurs that may need a 
 * recalculation - when a model object is deleted or
 * similar.  Sublasses should overide as appropriate
 */
protected void notifyChange() {
	queueCategory.notifyChange();
	channelCategory.notifyChange();
	if (null != messageResolution) {
		__continueMessageRoute(messageResolution, targetQueueManagerName, targetQueueName);
		messageResolution.calculateLines();
	}
}
void removeChannel(MQChannel channel) {
	queueCategory.removeChild(channel);
}
void removeQueue(MQQueue queue) {
	queueCategory.removeChild(queue);
}
public void requestDelete() {
	host.removeMQQueueManager(this);
	delete();
}
public void setChildrenVisible() {
	super.setChildrenVisible();
	queueCategory.setChildrenVisible();
	channelCategory.setChildrenVisible();
}
public final void setHost(Host host) {
	this.host = host;
}
}
