package designer.model;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import java.awt.Color;

public class QueueManagerEntry extends LogicalQueue {
	protected static final Color ALIAS_COLOR = new Color(255,200,255);
	StoreAndForwardQueue parent;
public QueueManagerEntry(String name, StoreAndForwardQueue parent) {
	super();
	setName(name);
	this.parent = parent;
}
/**
 * Resolve the put message route.  This is done by adding steps to the route
 * until the destination is reached (or unreachable
 *
 * Most subclasses will be quite content with the creation method used here 
 * - the step representing the receiver will be returned for the benefit of the
 * caller.  If the receiver wishes to propagate the resolution it should
 * provide an alternate implementation of this method.
 */
AbstractStep _resolveMessageRoute(AbstractStep previousStep, String qmName, String qName) {
    AbstractStep step = previousStep;
    step = super._resolveMessageRoute(previousStep, qmName, qName);
    getQueue()._resolveMessageRoute(step, qmName, qName);
    return step;
}
protected final AdministrationInterface getAdministrator() {
	return parent.getAdministrator();
}
public String getDescription() {
	return getShortDescription() + " on " + parent.getDescription();
}
public String getExplanation() {
	StringBuffer buf = new StringBuffer();
	buf.append(getShortDescription());
	buf.append(" allows ");
	buf.append(parent.getShortDescription());
	buf.append(" to accepts messages destined for any queue on queue manager named ");
	buf.append(getName());
	buf.append(".");
	return buf.toString();
}
/**
 * reply the model to which the receiver belongs
 */
protected final MQeModel getModel() {
	return parent.getModel();
}
public Model getParent() {
	return parent;
}
public final StoreAndForwardQueue getQueue() {
	return parent;
}
public final QueueManager getQueueManager() {
	return parent.getQueueManager();
}
public String getType() {
	return "Queue Manager Entry";
}
public boolean isQueueManagerEntry() {
	return true;
}
/**
 * Called to request that the receiver removes itself
 * from model
 */
public final void performDelete() {
	((StoreAndForwardQueue)parent).removeQueueManagerEntry(this);
	declareDeleted();
}
}
