package designer.model;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/
public class RemoteQueue extends AbstractRemoteQueue {
	private boolean synchronous = true;
	private PushMessageRoute pushMessageRoute;
    private DefaultRelationship resolvedQueue = null;
/**
 * RemoteQueue constructor comment.
 * @param owner snoop.model.QueueManager
 */
public RemoteQueue(QueueManager owner) {
	super(owner);
}
/**
 * Resolve the put message route.  This is done by adding steps to the route
 * until the destination is reached (or unreachable
 *
 * Most subclasses will be quite content with the creation method used here 
 * - the step representing the receiver will be returned for the benefit of the
 * caller.  If the receiver wishes to propagate the resolution it should
 * provide an alternate implementation of this method.
 */
AbstractStep _resolveMessageRoute(AbstractStep previousStep, String qmName, String qName) {
    AbstractStep step = super._resolveMessageRoute(previousStep, qmName, qName);
    Connection connection = getConnection();
    if (null != connection) {
	    connection._resolveMessageRoute(step, qmName, qName);
    }
    return step;
}
/**
 * delete this node if not already deleted
 */
public void delete() {
	if (null != resolvedQueue) {
		resolvedQueue.detach();
	}
    if (null != pushMessageRoute) {
        getQueueManager().removeMessageRoute(pushMessageRoute);
	    pushMessageRoute.delete();
        pushMessageRoute = null;
    }
	super.delete();
}
public String getExplanation() {
	StringBuffer buf = new StringBuffer();
	buf.append(getShortDescription());
	buf.append(" defines the means by which ");
	buf.append(getQueueManager().getShortDescription());
	buf.append(" accepts messages destined for a queue named ");
	buf.append(getQueueName());
	buf.append(" located on ");
	buf.append(getQueueQueueManagerName());
	buf.append(".");
			// TODO: comment on current connectivity....
	return buf.toString();
}
public String getType() {
	return "Remote Queue";
}
public final boolean isRemoteQueue() {
	return true;
}
public final boolean isSynchronous () {
	return synchronous;
}
/**
 * Method caled when a change occurs that may need a 
 * recalculation - when a model object is deleted or
 * similar.  Sublasses should overide as appropriate
 */
protected void notifyChange() {
    super.notifyChange();
    if (null != resolvedQueue && !resolvedQueue.isAttached()) {
        resolvedQueue = null;
    }
    if (null != pushMessageRoute && !pushMessageRoute.isValid()) {
        getQueueManager().removeMessageRoute(pushMessageRoute);
	    pushMessageRoute.delete();
        pushMessageRoute = null;
    }
    if (null == pushMessageRoute) {
        // if we have no route then calculate it
        Connection conn = getQueueManager().getConnection(getQueueQueueManagerName());
        if (null != conn) {
	        conn = conn.resolve();	// may be a via connection
        }
	    if (null != conn) {
            Listener listener = conn.resolveListener();
            if (null != listener) {
                QueueManager dstQM = listener.getQueueManager();
                LogicalQueue dstQueue = dstQM.resolveQueue(getQueueQueueManagerName(), getQueueName());
                if (null == dstQueue) {
	                dstQueue = dstQM.getStoreAndForwardQueueFor(getQueueQueueManagerName());
                }
                if (null != dstQueue) {
                    pushMessageRoute = new PushMessageRoute(this, conn, listener, dstQueue);
                    getQueueManager().addMessageRoute(pushMessageRoute);
                    if (!isToBeDeleted() && null == resolvedQueue) {
                        resolvedQueue = new DefaultRelationship(this, dstQueue, "resolves to");
                    }
                }
            }
        }
    }

    if (null != resolvedQueue) {
        resolvedQueue.setVisible(showDefault && ShowDefaultRelationships);
    }
}
public final void setSynchronous (boolean sync) {
	synchronous = sync;
}
}
