package designer.model;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import java.util.Vector;
import java.util.Enumeration;
import java.awt.PopupMenu;
import java.awt.Menu;
import java.awt.MenuItem;
import java.awt.event.ActionEvent;

/**
 * Insert the type's description here.
 * Creation date: (28/08/2001 09:48:03)
 * @author: 
 */
public class StoreAndForwardQueue extends AbstractRemoteQueue {
	private static final String MI_ADD_QUEUEMANAGER = "Add QueueManager";
	private final Category queueManagerCategory = new Category(this, "Targets");
	private MultiMessageRoute pushMessageRoute;
    private DefaultRelationship resolvedQueue = null;
/**
 * StoreAndForwardQueue constructor comment.
 * @param owner snoop.QueueManager
 */
public StoreAndForwardQueue(QueueManager owner) {
    super(owner);
}
/**
 * Resolve the put message route.  This is done by adding steps to the route
 * until the destination is reached (or unreachable
 *
 * Most subclasses will be quite content with the creation method used here 
 * - the step representing the receiver will be returned for the benefit of the
 * caller.  If the receiver wishes to propagate the resolution it should
 * provide an alternate implementation of this method.
 */
AbstractStep _resolveMessageRoute(AbstractStep previousStep, String qmName, String qName) {
    if (null != previousStep.nextStep && this != previousStep.nextStep.model) {
        previousStep.nextStep.detach();
        previousStep.nextStep = null;
    }

    if (null == previousStep.nextStep) {
        // need to create new step
        SaFStep step = new SaFStep(this);
        step.setTarget(qmName, qName);
        previousStep.setNextStep(step);
    }

    SaFStep step = (SaFStep) previousStep.nextStep;
    Connection connection = getConnection();
    if (null != connection) {
        connection._resolveMessageRoute(step, qmName, qName);
    }

    QueueManager destQM = getModel().getQueueManager(qmName);
    if (null != destQM) {
        HomeServerQueue hsq = destQM.findHomeServerQueueFor(getQueueManager().getName());
        if (null != hsq) {
            connection = hsq.getConnection();
            if (null != connection) {
                Listener listener = connection.resolveListener();
                if (null != listener) {
                    if (listener.getQueueManager() == getQueueManager()) {
                        // yes, we have a pull route"
                        PullStep listenerStep = new PullStep(listener);
                        listenerStep.setTarget(qmName, qName);
				        step.setPullStep(listenerStep);

				        PullStep connectionStep = new PullStep(connection);
				        connectionStep.setTarget(qmName, qName);
				        listenerStep.setNextStep(connectionStep);
				         
				        PullStep hsqStep = new PullStep(hsq);
				        hsqStep.setTarget(qmName, qName);
				        connectionStep.setNextStep(hsqStep);

				        hsq._resolveMessageRoute(hsqStep, qmName, qName);
                    }
                }
            }
        }
    }

    return step;
}
public final boolean acceptsMessagesFor(String queueManagerName) {
    boolean found = false;
    if (null != queueManagerCategory.getChild(queueManagerName)) {
        found = true;
    }
    return found;
}
public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();
    if (MI_ADD_QUEUEMANAGER.equals(cmd)) {
        String data[][] = new String[][] { { "QueueManager Name", "RemoteQM" }};
        if (getView().getStringData("Add Queue Manager Entry", data)) {
            addQueueManagerEntry(data[0][1]);
        }
    } else {
        super.actionPerformed(e);
    }
}
public final QueueManagerEntry addQueueManagerEntry(String queueManagerEntryName) {
	QueueManagerEntry qme = null;
	if (null == getQueueManagerEntry(queueManagerEntryName)) {
		qme = new QueueManagerEntry(queueManagerEntryName, this);
		queueManagerCategory.addNode(qme);
		qme.commit();
	}
	return qme;
}
/**
 * Commit all changes for receiver and receivers children
 */
public void commitAll() {
	super.commitAll();
	queueManagerCategory.commitAll();
}
/**
 * delete this node if not already deleted
 */
public void delete() {
    queueManagerCategory.delete();
    if (null != pushMessageRoute) {
        getQueueManager().removeMessageRoute(pushMessageRoute);
        pushMessageRoute.delete();
        pushMessageRoute = null;
    }
    super.delete();
}
public String getExplanation() {
	StringBuffer buf = new StringBuffer();
	buf.append(getShortDescription());
	buf.append(" defines the means by which ");
	buf.append(getQueueManager().getShortDescription());
	buf.append(" accepts messages destined for any queue on one of the defined queue managers. ");
	Vector entries = getQueueManagerEntries();
	if (entries.size() > 0) {
		buf.append("The following queue managers are currently defined:\n");
		for (int i = 0; i < entries.size(); i++){
			buf.append("\t-\t");
			QueueManagerEntry qme = (QueueManagerEntry)entries.elementAt(i);
			buf.append(qme.getName());
			buf.append("\n");
		}
	} else {
		buf.append("There are currently no defined queue managers, so the Store and forward queue will accept no messages. ");
	}
	if (getQueueQueueManagerName().equals(getQueueManager().getName())) {
		buf.append("\n\nThis store and forward queue is set up as a store queue because the destination ");
		buf.append("queue manager name is the same as the owning queue manager name.  This means that ");
		buf.append("messages will only move from the queue when they are pulled by a Home Server Queue. ");
	} else {
		buf.append("\n\nThis store and forward queue is set up both as a store queue and as a forward queue ");
		buf.append("because the destination ");
		buf.append("queue manager name is not the same as the owning queue manager name.  This means that ");
		buf.append("messages can move from the queue either when they are pulled by a Home Server Queue, ");
		buf.append("or when this queue pushes them to ");
		QueueManager destQM = getModel().getQueueManager(getQueueQueueManagerName());
		if (null == destQM) { 	
			buf.append(destQM.getShortDescription());
			// TODO: comment on current connectivity....
		} else {
			buf.append("a queue manager called ");
			buf.append(getQueueQueueManagerName());
			buf.append(" which currently does not exist");
		}
		buf.append(".");
	}
	buf.append(".");
	return buf.toString();
}
public PopupMenu getMenu() {
	PopupMenu popup = super.getMenu();
	MenuItem menuItem = new MenuItem(MI_ADD_QUEUEMANAGER);
	menuItem.addActionListener(this);
	popup.add(menuItem);
	
	return popup;
}
public final Vector getQueueManagerEntries() {
	return queueManagerCategory._getChildren();
}
public final QueueManagerEntry getQueueManagerEntry(String queueManagerEntryName) {
	return (QueueManagerEntry)queueManagerCategory.getChild(queueManagerEntryName);
}
public String getType() {
	return "Store And Forward Queue";
}
/**
 * reply true if the receiver is a pushing store and forward queue
 */
public final boolean isPushing() {
	if (getQueueManager().getName().equals(getQueueQueueManagerName())) {
		// our queue manager name is the same as our owning qm, therefore
		// we are not pushing
		return false;
	}
	return true;
}
public boolean isStoreAndForwardQueue() {
	return true;
}
/**
 * Method caled when a change occurs that may need a 
 * recalculation - when a model object is deleted or
 * similar.  Sublasses should overide as appropriate
 */
protected void notifyChange() {
    super.notifyChange();
    if (null != resolvedQueue && !resolvedQueue.isAttached()) {
        resolvedQueue = null;
    }
     if (null != pushMessageRoute) {
        // if we have a push route, and it is invalid
        // then remove it
        if (!pushMessageRoute.isValid()) {
            getQueueManager().removeMessageRoute(pushMessageRoute);
            pushMessageRoute.delete();
            pushMessageRoute = null;
        }
    }
    if (null == pushMessageRoute && getQueueManagerEntries().size() > 0) {
        // if we have no route then calculate it
        Connection conn = getQueueManager().getConnection(getQueueQueueManagerName());
        if (null != conn) {
            Listener listener = conn.resolveListener();
            if (null != listener) {
                QueueManager dstQM = listener.getQueueManager();
                //Queue dstQueue = dstQM.getQueue(getQueueQueueManagerName(), getQueueName());
                if (null != dstQM && dstQM != getQueueManager()) {
                    pushMessageRoute = new MultiMessageRoute(this, conn, listener, dstQM);
                    getQueueManager().addMessageRoute(pushMessageRoute);
                    if (!isToBeDeleted() && null == resolvedQueue) {
                        resolvedQueue = new DefaultRelationship(this, dstQM, "resolves to");
                    }
                }
            }
        }
    }
    if (null != resolvedQueue) {
        resolvedQueue.setVisible(showDefault && ShowDefaultRelationships);
    }
}
public final void removeQueueManagerEntry(QueueManagerEntry queueManagerEntry) {
	queueManagerCategory.removeChild(queueManagerEntry);
}
public final boolean removeQueueManagerEntry(String queueManagerEntryName) {
	boolean changed = false;
	QueueManagerEntry qme = getQueueManagerEntry(queueManagerEntryName);
	if (null != qme) {
		qme.requestDelete();
		qme.commit();
	}
	return changed;
}
public final LogicalQueue resolve(String queueManagerName, String queueName, boolean allowSAFEntries) {
	LogicalQueue found = null;
	if (allowSAFEntries) {
		found = (LogicalQueue)queueManagerCategory.getChild(queueManagerName);
	}
	return found;
}
public void setChildrenVisible() {
	super.setChildrenVisible();
	queueManagerCategory.setChildrenVisible();
}
}
