package glyph;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import java.awt.*;
import java.util.*;
class AreaGlyph extends Object {
	
	private static final int HOTSPOTSIZE = 4;
	private static final int HOTSPOT_NONE = 0;
	private static final int HOTSPOT_EXPANDED = 1;
	private static final int HOTSPOT_CONTRACTED = 2;

	private static final int SHADOW = 4;
	private static final int CHILD_SPACING = 4;
		
	private AreaGlyph parent = null;
	/** logical xCoordinate (ie top left when not selected)*/
	private int xCoord = 100;
	/** logical yCoordinate (ie top left when not selected)*/
	private int yCoord = 100;
	private int width;
	private int height;
	private int textHeight;
	private int textDescent;
	
	/** logical xCoordinate (ie top left may be modified by being selected)*/
	int physicalXCoord;
 	/** logical xCoordinate (ie top left may be modified by being selected)*/
	int physicalYCoord;
    
	private boolean expanded = false;
	private boolean showBoundary = false;
	private int hotspotX;
	private int hotspotY;
	private int nameY;

	private final Vector children = new Vector();
	private final Vector attachedLines = new Vector();
	
	private boolean _detatched = false;
	private boolean changed = true;
	
	Node node = null;

	private boolean selected = false;

	private View glyphView;

	private Aggregate aggregate = null;

	private boolean isDropTarget = false;		// special draw of parent if child dragged over

/**
 * Sole constructor is available only to package.
 * By insisting on a node value we save ourselves 
 * from having continually to check for null.
 */
AreaGlyph(Node node) {
	super();
	this.node = node;
}
/**
 * return width of string representation of this glyph
 */
private final int __calculateBody(FontMetrics fontMetrics) {
    int bodyWidth = 0;
    String[] txt = node.getBody();
    height = 0;
    if (null != txt) {
        for (int i = 0; i < txt.length; i++) {
            String str = txt[i];
            if (null != str) {
                height += fontMetrics.getHeight();
                int w = fontMetrics.stringWidth(str);
                if (w > bodyWidth) {
                    bodyWidth = w;
                }
            }
        }
    }
    String heading = getHeading();
    if (isDetached() && null != heading) {
        int headingWidth = fontMetrics.stringWidth(heading);
        if (headingWidth > bodyWidth) {
            bodyWidth = headingWidth;
        }
    }
    return bodyWidth + 4;
}
/**
 * the receiver has changed in a way that requires recalculation
 * before the next repaint.
 * Note that this call is propagated down and up the tree
 */
final void __changed() {
    if (!changed) {
        changed = true;
        for (int i = 0; i < attachedLines.size(); i++) {
            ((LineGlyph) attachedLines.elementAt(i))._changed();
        }
        if (null != parent && !isDetached()) {
            parent.__changed();
        }
    }
    for (int i = 0; i < children.size(); i++) {
        AreaGlyph glyph = (AreaGlyph) children.elementAt(i);
        glyph.__changed();
    }
}
/**
 * draw the hotspot onto the graphics provided.
 * we do this after all the receivers siblings have
 * been drawn, because the algorithm is simpler. 
 */
private final int __drawHotspot(Graphics g, int type, int hotspotX, int hotspotY) {
	//int fontHeight = g.getFontMetrics().getHeight();
	if (HOTSPOT_NONE == type) {
		g.fillOval(hotspotX - HOTSPOTSIZE, hotspotY - HOTSPOTSIZE,
					2*HOTSPOTSIZE, 2*HOTSPOTSIZE);
	} else {
		int x1 = hotspotX - HOTSPOTSIZE;
		int x2 = hotspotX + HOTSPOTSIZE;
		int y1 = hotspotY - HOTSPOTSIZE;
		int y2 = hotspotY + HOTSPOTSIZE;

		g.setColor(Color.white);
		g.fillRect(x1, y1, 2 * HOTSPOTSIZE, 2 * HOTSPOTSIZE);
		g.setColor(Color.black);
		g.drawRect(x1, y1, 2 * HOTSPOTSIZE, 2 * HOTSPOTSIZE);
		x1 += 2;
		x2 -= 2;
		g.drawLine(x1, hotspotY, x2, hotspotY);
		if (HOTSPOT_CONTRACTED == type) {
			y1 += 2;
			y2 -= 2;
			g.drawLine(hotspotX, y1, hotspotX, y2);
		}
	}
	return textHeight;
}
final void __setOrigin(int x, int y) {
	xCoord = x;
	yCoord = y;
}
/**
 * add any detached glyphs to the specified vector.
 * The vector is then used to determine the drawing 
 * order for the whole display
 *
 * This method is deliberately package private.  It should
 * only be called by the view.
 */
void _addDetachedGlyphsTo(Vector vector) {
	if (isDetached()) {
		vector.addElement(this);
	}
	for (int i = 0; i < children.size(); i++){
		((AreaGlyph)children.elementAt(i))._addDetachedGlyphsTo(vector);
	}
}
protected final void _addLine(LineGlyph line) {
	if (!attachedLines.contains(line)) {
		attachedLines.addElement(line);
		if (null != glyphView) {
			glyphView._addLine(line);
		}
	}
}
/**
 * we need do this only for detached glyphs
 */
void _calculate(Graphics graphics, Rectangle bounds) {
    if (changed) {
        FontMetrics fontMetrics = graphics.getFontMetrics();
        textHeight = fontMetrics.getHeight();
        textDescent = fontMetrics.getDescent();
        physicalXCoord = xCoord;
        physicalYCoord = yCoord;
        if (node.canDetach() && selected || isDropTarget) {
            physicalXCoord -= SHADOW;
            physicalYCoord -= SHADOW;
        }
        
        hotspotX = physicalXCoord + (textHeight / 2);
        hotspotY = physicalYCoord + (textHeight / 2);
        // add on space for hotspot AND for trailing space
        int leftMargin = physicalXCoord + textHeight;
        width = __calculateBody(fontMetrics) + textHeight;
        nameY = physicalYCoord + (textHeight - textDescent);
        int currentTop = physicalYCoord + height;
        if (null != getHeading() && isDetached()) {
            currentTop += textHeight;
            hotspotY += textHeight;
            nameY += textHeight;
            height += textHeight;
        }
        // draw children
        int maxChildWidth = 0;
        for (int i = 0; i < children.size(); i++) {
            AreaGlyph glyph = (AreaGlyph) (children.elementAt(i));
            if (!glyph.isDetached() && expanded && glyph.node.isVisible()) {
                glyph.__setOrigin(leftMargin, currentTop);
                glyph._calculate(graphics, bounds);
                if (glyph.isWorthDrawing()) {
                    int glyphHeight = glyph.getHeight();
                    if (0 < glyphHeight) {
                        height += glyphHeight + CHILD_SPACING;
                        currentTop += (glyphHeight + CHILD_SPACING);
                        // has width increased
                        int glyphWidth = glyph.getWidth();
                        if (maxChildWidth < glyphWidth) {
                            maxChildWidth = glyphWidth;
                        }
                    }
                }
            }
        }
        if (maxChildWidth > 0) {
            // only if we have a child width
            // add width of tree lines
            maxChildWidth += textHeight;
            if (width < maxChildWidth) {
                width = maxChildWidth;
            }
            width += 4;
        }
    }
    bounds.add(physicalXCoord, physicalYCoord);
    bounds.add(physicalXCoord + width, physicalYCoord + height);
    changed = false;
}
/**
 * definitely package use only....
 */
final Vector _children() {
	return children;
}
final boolean _drag(Point start, Point end) {

	if (node.canDetach()) {
		xCoord = xCoord + end.x - start.x;
		yCoord = yCoord + end.y - start.y;
		if (!isDetached()) {
			setDetatched(true);
			node.detached();
		}
		if (null != parent) {
	        boolean isOverDropTarget = _isOverDropTarget();
	        parent.highestAttachedGlyph()._setIsDropTarget(isOverDropTarget);
	    }
		__changed();
	} else if (!node.canDetach()) {
		if (null != parent) {
			parent._drag(start, end);
		}
	}
	return true;
}
final boolean _dragEnd(Point pt) {
	boolean changed = false;
	if (contains(pt)) {
		if (null == parent) {
			setDetatched(true);
		} else {
			setDetatched(!_isOverDropTarget());
			if (!isDetached()) {
				node.attached();
			}
			highestAttachedGlyph()._setIsDropTarget(false); 
		}
		__changed();
		changed = true;
	}
	return changed;
}
void _draw(DrawContext graphics) {
    if (!node.isVisible()) {
        return;
    }
    if (!isSetVisible()) {
	    return;
    }
    int hotspotType = HOTSPOT_NONE;
    boolean hasChildren = false;
    for (int i = 0; i < children.size(); i++) {
        AreaGlyph glyph = (AreaGlyph) (children.elementAt(i));
        if (!glyph.isDetached()) {
            if (glyph.isWorthDrawing() && glyph.node.isVisible()) {
                hasChildren = true;
            }
        } 
    }
    if (hasChildren) {
        if (expanded) {
            hotspotType = HOTSPOT_EXPANDED;
        } else {
            hotspotType = HOTSPOT_CONTRACTED;
        }
    }
    if (isDetached()) {
        graphics.drawRect(physicalXCoord, physicalYCoord, width, height, Color.black, Color.white);
        String heading = getHeading();
        if (null != heading) {
            graphics.drawRect(physicalXCoord, physicalYCoord, width, textHeight, Color.black, Color.lightGray);
            graphics.drawString(heading, physicalXCoord + (textHeight / 2), physicalYCoord + (textHeight - textDescent));
        }
    } else if (null != parent) {
        //if (node.canDetach()) {
        //    graphics.drawRect(physicalXCoord, physicalYCoord, width, height);
        //}
        graphics.drawLine(parent.hotspotX, parent.hotspotY + HOTSPOTSIZE, parent.hotspotX, hotspotY);
        int lineEndX = hotspotX;
        if (HOTSPOT_NONE != hotspotType) {
            lineEndX -= HOTSPOTSIZE;
        }
        graphics.drawLine(parent.hotspotX, hotspotY, lineEndX, hotspotY);
    }
    if (HOTSPOT_NONE != hotspotType) {
        int x1 = hotspotX - HOTSPOTSIZE;
        int x2 = hotspotX + HOTSPOTSIZE;
        int y1 = hotspotY - HOTSPOTSIZE;
        int y2 = hotspotY + HOTSPOTSIZE;
        graphics.drawRect(x1, y1, 2 * HOTSPOTSIZE, 2 * HOTSPOTSIZE, Color.black, Color.white);
        x1 += 2;
        x2 -= 2;
        graphics.drawLine(x1, hotspotY, x2, hotspotY);
        if (HOTSPOT_CONTRACTED == hotspotType) {
            y1 += 2;
            y2 -= 2;
            graphics.drawLine(hotspotX, y1, hotspotX, y2);
        }
    }
    // keep a reference to the Y coord of the current text origin
    String[] txt = node.getBody();
    if (null != txt) {
        int tempY = nameY;
        //int textHeight = fontMetrics.getHeight();
        for (int i = 0; i < txt.length; i++) {
            String str = txt[i];
            if (null != str) {
                graphics.drawString(str, physicalXCoord + textHeight, tempY);
                tempY += textHeight;
            }
        }
    }
    // draw children
    if (expanded) {
        for (int i = 0; i < children.size(); i++) {
            AreaGlyph glyph = (AreaGlyph) (children.elementAt(i));
            if (!glyph.isDetached() && glyph.isWorthDrawing() && glyph.node.isVisible()) {
                glyph._draw(graphics);
            }
        }
    }
}
void _drawBoundary(DrawContext graphics) {
    if (showBoundary) {
        Boundary boundary = getBoundary(null);
        if (null != boundary) {
	       boundary._draw(graphics, node.getShortDescription(), textDescent);
        }
    }
}
void _drawShadow(DrawContext graphics) {
    if (!node.isVisible()) {
        return;
    }
    // shadow along bottom
    int shadowLeft = physicalXCoord + SHADOW;
    int shadowTop = physicalYCoord+height;
    graphics.drawRect(shadowLeft, shadowTop, width, SHADOW, Color.black, Color.black);
	// shadow down right side
	shadowLeft = physicalXCoord+width;
	shadowTop = physicalYCoord +SHADOW;
    graphics.drawRect(shadowLeft, shadowTop, SHADOW, height, Color.black, Color.black);
	
}
/**
 * reply true if the receiver is currently over its parent, or over its parents
 * highest attached glyph)
 */
private final boolean _isOverDropTarget() {
    boolean isOver = false;
    if (null != parent) {
        AreaGlyph target = parent.highestAttachedGlyph();
        if (target.isSetVisible()) {
            if (xCoord > target.xCoord + target.width) {
                // receiver left is to right of target right - not included
            } else if (yCoord > target.yCoord + target.height) {
                // receiver top is below target bottom - not included
            } else if (target.xCoord > xCoord + width) {
                // target left is to right of receiver right - not included
            } else if (target.yCoord > yCoord + height) {
                // target top is below receiver bottom - not included
            } else {
                isOver = true;
            }
        }
    }
    return isOver;
}
protected final void _removeLine(LineGlyph line) {
	attachedLines.removeElement(line);
	if (null != glyphView) {
		glyphView._removeLine(line);
	}
}
final void _setIsDropTarget(boolean set) {
	isDropTarget = set;
	__changed();
}
/**
 * Set the receivers parent
 * Package visibility intended, as should only be called by
 * another AreaGlyph
 */
private final void _setParent(final AreaGlyph parent) {
	if (null != this.parent) {
		this.parent.__changed();
	}
	if (parent != this.parent) {
		if (null != aggregate) {
			aggregate.delete();
			aggregate = null;
		}
	}
	this.parent = parent;
	if (null == parent) {
		setDetatched(true);
	} else {
	    if (node.canDetach() && null == aggregate) {
		    //if(((designer.model.Model)parent.getNode()).isQueueManager()) {
			//    System.out.println("adding qm child");
		    //}
	        aggregate = new Aggregate(this.getNode(), parent.getNode());
	    }
		//setDetatched(false);
		parent.__changed();
	}
	__changed();
}
/**
 * Set the receivers view.  Package level visibility is intentional
 * as this should all be hidden from the user
 */
void _setView(View view) {
	glyphView = view;
	if (null != glyphView) {
		for (int i = 0; i < attachedLines.size(); i++) {
			glyphView._addLine((LineGlyph) (attachedLines.elementAt(i)));
		}
	}
	for (int i = 0; i < children.size(); i++) {
		AreaGlyph glyph = (AreaGlyph) children.elementAt(i);
		glyph._setView(glyphView);
	}
}
public final void addChild(AreaGlyph child) {
	if (null != child) {
		if (!children.contains(child)) {
			children.addElement(child);
		}
		child._setParent(this);
		child._setView(glyphView);
		repaint();
	}
}
/**
 * deliberate package private
 */
final Vector attachedLines() {
	return attachedLines;
}
/**
 * Should a child of the receiver reattach to the receiver
 * when dragging ends at the given point.
 * If the receiver is attached its 'capture area' is its parents
 * area, otherwise the capture area is its own area. 
 */
public final boolean canAttachChildAt(Point pt) {
	boolean can = false;
	if (isDetached() || null == parent) {
		can = contains(pt);
	} else {
		can = parent.canAttachChildAt(pt);
	}
	return can;
}
public boolean click(Point pt) {
	if (isVisible() && isHotspot(pt.x,pt.y)) {
		setExpanded(!expanded);
		return true;
	}
	return false;
}
public final boolean contains(int x, int y) {
	if (isVisible()  && isSetVisible()) {
		if (x < xCoord || x > xCoord + width
	        || y < yCoord || y > yCoord + height) {
	            return false;
	    }
		return true;
	}
	return false;
}
public boolean contains(Point pt) {
	return contains(pt.x, pt.y);
}
final void delete() {
	if (null == parent) {
		if (null != glyphView) {
			glyphView._remove(this);
		}
	} else {
		parent.removeChild(this);
	}
	
	LineGlyph[] lines = new LineGlyph[attachedLines.size()];
	attachedLines.copyInto(lines);
	// all our lines are detached
	for (int i = 0; i < lines.length; i++){
		lines[i].detach();
	}
	attachedLines.removeAllElements();
	
	AreaGlyph[] areas = new AreaGlyph[children.size()];
	children.copyInto(areas);
	// all our children are deleted
	for (int i = 0; i < areas.length; i++){
		areas[i].delete();
	}
}
public void deselect() {
	selected = false;
	__changed();
}
public void deselectAll() {
	deselect();
	for (int i = 0; i < children.size(); i++){
		((AreaGlyph)children.elementAt(i)).deselectAll();
	}
}
/**
 * Reply the glyph associated with the specified node
 * or null if the glyph cannot be located.  Only children of
 * the receiver (and their children recursively) will be searched.
 */
final AreaGlyph findGlyphFor(Node node) {
	AreaGlyph found = null;
	if (null != node) {
		if (this.node == node) {
			found = this;
		} else {
			for (int i = 0; null == found && i < children.size(); i++){
				AreaGlyph glyph = (AreaGlyph)children.elementAt(i);
				found = glyph.findGlyphFor(node);
			}
		}
	}
	return found;
}
final void forceChildrenVisible() {
	for (int i = 0; i < children.size(); i++){
		AreaGlyph glyph = (AreaGlyph)children.elementAt(i);
		glyph.node.setVisible(true);
	}
}
/**
 * Augment the size of the given rectangle to include
 * the receiver
 */
public Boundary getBoundary(Boundary boundary) {
	for (int i = 0; i < children.size(); i++){
		AreaGlyph glyph = (AreaGlyph)children.elementAt(i);
		boundary = glyph.getBoundary(boundary);
		if (glyph.getShowBoundary()) {
			// if the contained child is showing a boundary
			// we need to add some space in for the childs
			// boundary text
			boundary.addTextHeight(textHeight);
		}
	}
	if (isSetVisible() && isDetached()) {
		if (null == boundary) {
			boundary = new Boundary(physicalXCoord, 
	                physicalYCoord, 
	                physicalXCoord + width, 
	                physicalYCoord + height);
		} else {
			boundary.include(physicalXCoord, physicalYCoord, physicalXCoord+width, physicalYCoord+height);
		}
	}
	if (getShowBoundary()) {
		// if we are going to show a boundary then we need to add some extra
		// so the lines do not overlap
		boundary.enlarge();
	}
	return boundary;
}
public final Point getCentre() {
	if (null != parent && !isDetached()) {
		return parent.getCentre();
	}
	return new Point(xCoord + (width / 2), yCoord + (height / 2));
}
/**
 * return number of parents above receiver
 */
public final int getDepth() {
	int depth = 0;
	if (null != parent) {
		depth = 1 + parent.getDepth();
	}
	return depth;
}
public String getDisplayName() {
	String name = "";
	if (null != node) {
		String[] txt = node.getBody();
		if (null != txt && txt.length > 0) {
			name = txt[0];
		}
	}
	return name;
}
public final AreaGlyph getGlyph(Point pt) {
	AreaGlyph found = null;
	// first check if it is one of our children
	Enumeration enum = children.elements();
	while (/*null == found &&*/ enum.hasMoreElements()) {
		AreaGlyph glyph = (AreaGlyph) enum.nextElement();
		//if (expanded || glyph.isDetached()) {
			AreaGlyph g = glyph.getGlyph(pt);
			if (null != g) {
				found = g;
			}
		//}
	}
	if (null == found && contains(pt) && isVisible() && isWorthDrawing()) {
	    found = this;
	}
	return found;
}
public final View getGlyphView() {
	return glyphView;
}
/**
 * Return the type of this object - to be displayed at top of glyph
 * if the glyph is detatched.  Return null to suppress this.
 */
protected String getHeading() {
	if (null != node) {
		return node.getHeader();
	}
	return null;
}
public final int getHeight() {
	return height;
}
public Point getIntersectPointFrom(Point source) {
	if (null != parent && !isDetached()) {
		return parent.getIntersectPointFrom(source);
	}

	// coordinates of logical center of rectangle should not change when
	// the glyph is selected.
	int centreX = xCoord + (width / 2);
	int centreY = yCoord + (height / 2);
	// calculate the slope of the line as dX/dY
	int deltaY = source.y - centreY;
	int deltaX = source.x - centreX;
	double slope = ((double) deltaX) / Math.abs(deltaY);
	// calculate where the line will intersect the horizontal 
	// boundary of the rectangle.
	int pointX = centreX + (int) (height * slope) / 2;
	int pointY = centreY;

	if (pointX >= xCoord && pointX <= xCoord + width) {
		pointY = yCoord;
		if (source.y > centreY) {
			pointY += height;
		}
//		if (selected || isDropTarget) {
//			if (View.Show3d) {
				// allow for selection displacement
//				pointY -= SELECT_JUMP;
//			}
//		}
	} else {
		// intersection point at y boundary is outside width, so
		// calculate intersection at x boundary
		slope = ((double) deltaY) / Math.abs(deltaX);
		pointY = pointY + (int) (width * slope) / 2;
		if (pointY >= yCoord && pointY <= yCoord + height) {
			// intersection point at y boundary is within height
			pointX = xCoord;
			if (source.x > centreX) {
				pointX += width;
			}
//			if (selected || isDropTarget) {
//				if (View.Show3d) {
					// allow for selection displacement
//					pointX -= SELECT_JUMP;
//				}
//			}
		}
	}

	return new Point(pointX, pointY);
}
protected int getMaxXvalue() {
	return xCoord+width;
}
public final PopupMenu getMenu() {
	PopupMenu popup = null;
	if (null != node) {
		popup = node.getMenu();
	}
	return popup;
}
final Node getNode() {
	return node;
}
/**
 * Get the receivers parent
 */
public final AreaGlyph getParent() {
	return parent;
}
final boolean getShowBoundary() {
	return showBoundary;
}
public final int getWidth() {
	return width;
}
final int getXcoord() {
	return xCoord;
}
final int getYcoord() {
	return yCoord;
}
public final boolean hasChildren() {
	return children.size() > 0;
}
/**
 * return the glyph highest in the hierarchy to which the 
 * receiver is attached
 */
final AreaGlyph highestAttachedGlyph() {
	AreaGlyph glyph = this;
	if (null != parent && !glyph.isDetached()) {
		glyph = parent.highestAttachedGlyph();
	}
	return glyph;
}
/**
 * return true if the receiver is attached to the passed glyph.
 * (two glyphs are considered attached if they are both attached
 * to a third)
 */
public boolean isAttachedTo(AreaGlyph glyph) {
	return highestAttachedGlyph() == glyph.highestAttachedGlyph();
}
public boolean isDetached() {
	if (null == parent) {
		return true;
	}
	return _detatched;
}
	//private boolean isOverDropTarget = false;		// special draw of parent if child dragged over
final boolean isDropTarget() {
	return isDropTarget;
}
public final boolean isExpanded() {
	return expanded;
}
/**
 * reply true if the point (described by the given cordinate)
 * lies within the hotspot area
 */
public final boolean isHotspot(int x, int y) {
	return 	x >= hotspotX - HOTSPOTSIZE
			&& x <= hotspotX + HOTSPOTSIZE
			&& y >= hotspotY - HOTSPOTSIZE
			&& y <= hotspotY + HOTSPOTSIZE;
}
public boolean isSelected() {
	return selected;
}
/**
 * reply true if the user has allowed this to be visible
 */
public boolean isSetVisible() {
	boolean setVisible = node.isVisible();
	if (null != parent && !isDetached()) {
		if (setVisible) {
			setVisible = parent.isSetVisible();
		}
	}
	return setVisible;
}
public boolean isVisible() {
	if (isDetached()) {
		return true;
	} else {
		if (null != parent && parent.isVisible() && parent.expanded) {
			return true;
		}
	}
	return false;
}
protected boolean isWorthDrawing() {
	boolean drawMe = node.drawIfNoChildren();
	for (int i = 0; !drawMe && i < children.size(); i++){
		AreaGlyph glyph = (AreaGlyph)(children.elementAt(i));
		if (!glyph.isDetached() && glyph.isWorthDrawing()) {
			drawMe = true;
		}
	}
	return drawMe;
}
public final void removeChild(AreaGlyph child) {
	children.removeElement(child);
	child._setParent(null);
	repaint();
}
protected final void repaint() {
	if (null != glyphView) {
		glyphView.repaint();
	}
}
public void select() {
	selected = true;
	__changed();
}
/**
 * replace the receivers list of children with a new list
 */
public void setChildren(Vector newChildren) {
	children.removeAllElements();
	Enumeration enum = newChildren.elements();
	while (enum.hasMoreElements()) {
		AreaGlyph glyph = (AreaGlyph)enum.nextElement();
		addChild(glyph);
	}
}
final void setCoords(int x, int y) {
	setDetatched(true);
	xCoord = x;
	yCoord = y;
}
public final void setDetatched(final boolean set) {
	if (set != _detatched) {
		_detatched = set;
		node.declareChanged();
	}
}
public final void setExpanded(boolean expanded) {
	this.expanded = expanded;
	node.expanded();
	__changed();
	repaint();
}
final void setShowBoundary(boolean show) {
	showBoundary = show;
}
public String toString() {
	return node.getShortDescription();
}
}
