package glyph;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import java.lang.String;
import java.io.*;
import java.net.*;
import java.awt.*;
import java.awt.image.*;
/**
 * This is a class that reads and writes a
 * BMP formatted file
 */
class ImageStorer {
	private static final int COLOURS = 256;
	private static final int BITSIZE = 8;
	private static IndexColorModel icm;
	String bfName;
	static int colors[] = new int[256];
static int _getColourIndex(int pix) {
	for (int i = 0; i < colors.length; i++){
		if (colors[i] == pix) {
			return i;
		}
	}
	return 255;
}
static void _mapColours(int[] pix) {
	// scan the pixel array and store the first 254 unique non-white colours
	// as elements 1-254 of the colour index
	int coloursSaved = 1;	// start saving at pos 1
	for (int i = 0; i < colors.length; i++){
		// default colours to white
		colors[i] = 0xffffffff;
	}
	colors[255] = 0xff000000;	// last one is black
	for (int i=0; i <pix.length; i++) {
		//scan all pixels
		int p = pix[i];
		if (0xff000000 != p && 0xffffffff != p) {
			// not black
			if (_getColourIndex(p) == 255) {
				//not found
				if (coloursSaved < 254) {
					// room to save another colour
					colors[coloursSaved] = p;
					coloursSaved += 1;
					if (coloursSaved >= 254) {
						// no more room for colours
						return;
					}
				}
			}
		} 
	}
}
/**
  * Write out integer to little endian stream...
  * @DataOutputStream os - The output stream to write
  * @int data - Data to convert
  * @int len - Length of array to convert 
  */
private static void _pushVal(DataOutputStream os, int data, int len) throws IOException {
	byte b[] = new byte[len];
	for (int i = 0; i < len; ++i) {
		b[i] = (byte) (data >> (i * 8));
	}
	os.write(b, 0, len);
}
/* Write out a bitmap...
  * Current only supporting 8 bits per pixel...
  * @String filename - The file to save it as...
  * @IndexColorModel ICM - Palette to use 
  * @int pix[] - Pixels to save
  * @int width - Width of data
 * @int height - Height of data
  */
private static void _saveBitmap(String filename, Image image, int width, int height) throws IOException {
	// Create output stream...

	int pix[] = new int[width * height];
	PixelGrabber pg = new PixelGrabber(image, 0, 0, width, height, pix, 0, width);
	try {
		pg.grabPixels();
	} catch (InterruptedException e) {
		System.out.println(e);
		return;
	}
	if ((pg.getStatus() & ImageObserver.ABORT) != 0) {
		System.err.println("image fetch aborted or errored");
		return;
	}
	// we have grabbed all the pixels as RGB values.  We must map these to index values
	// ie 0x00010203 = colour 1
	// and store these values into the ICM
	_mapColours(pix);
	
	//IndexColorModel icm = _getIcm(cm);
	DataOutputStream os = new DataOutputStream(new BufferedOutputStream(new FileOutputStream(filename)));
	_writeFileHeader(os, width, height);
	_write8bitWindowsHeader(os, width, height);
	_write8bitColorIndex(os);

	_write8bitData(os, pix, width, height);
	os.close();
}
/**
  * Write the bitmap header out to Windows... 
  * @DataOutputStream os - The output stream to write
  * @IndexColorModel ICM - Palette to use 
 */
private static void _write8bitColorIndex(DataOutputStream os) throws IOException {
	// Write out palette...
	for (int i = 0; i < colors.length; ++i) {
		int b = colors[i] & 0xff;
		int g = (colors[i] >> 8) & 0xff;
		int r = (colors[i] >> 16) & 0xff;
		os.writeByte(b); //blue
		os.writeByte(g); //green
		os.writeByte(r); //red
		os.writeByte(0);
	} // end for
}
/**
  * Write out data of bitmap...
  * @DataOutputStream os - The output stream to write
  * @IndexColorModel ICM - Palette to use 
  * @int pix[] - Pixels to save
  * @int width - Width of data
  * @int height - Height of data
  */
private static void _write8bitData(DataOutputStream os, int pix[], int width, int height) throws IOException {
	// Bytes b...
	byte b[] = new byte[width];
	for (int i = 0; i < 4; ++i)
		b[i] = 0;
	// Calculate padding
	int padding = 0;
	int overage = width % 4;
	if (overage != 0)
		padding = 4 - overage;
	// Write out starting from bottom of height
	int index, x, y;
	for (y = (height - 1); y >= 0; --y) {
		// Write out each row, send to big buffer...
		index = (y * width);
		for (x = 0; x < width; ++x, ++index) {
			b[x] = (byte) _getColourIndex(pix[index]);
		}
		// Write out a big block...
		os.write(b, 0, width);
		// Send out padding...
		if (padding != 0) {
			for (int i = 0; i < 4; ++i) {
				b[i] = 0;
			}
			os.write(b, 0, padding);
		}
	} // end y for
}
/**
  * Write the bitmap header out to Windows... 
  * @DataOutputStream os - The output stream to write
  * @IndexColorModel ICM - Palette to use 
  * @int width - Width of data
  * @int height - Height of data
 */
private static void _write8bitWindowsHeader(DataOutputStream os, int width, int height) throws IOException {
	_pushVal(os, 40, 4); // Bytes in header
	_pushVal(os, width, 4); // Size in pixels...
	_pushVal(os, height, 4);
	_pushVal(os, 1, 2); // # Color Planes
	_pushVal(os, 8, 2); // Bits per pixel...
	_pushVal(os, 0, 4); // NO Compression...
	_pushVal(os, width * height, 4); // Size of image...
	// LATER PUT IN REAL DATA pixels/meter
	_pushVal(os, 3790, 4); // TBD: Horizontal Res pixels/meter
	_pushVal(os, 3790, 4); // TBD: Vertical Res pixels/meter
	_pushVal(os, colors.length, 4); // Indexes in bitmap... 
	_pushVal(os, colors.length, 4); // Indexes in bitmap...
}
/**
  * Write out the file header
  * @DataOutputStream os - The output stream to write
  * @IndexColorModel ICM - Palette to use 
  * @int width - Width of data
 * @int height - Height of data
 */
private static void _writeFileHeader(DataOutputStream os, int width, int height) throws IOException {
	byte b[] = new byte[4];
	// Write out magic code... 
	b[0] = (byte) 'B';
	b[1] = (byte) 'M';
	os.write(b, 0, 2);

	// Calculate size and offset.. 
	int paletteSize = (colors.length * 4);
	int offset = 54 + paletteSize;
	int fileSize = offset + (width * height);

	// Write out size & offset... 
	_pushVal(os, fileSize, 4);
	_pushVal(os, 0, 4);
	_pushVal(os, offset, 4);
}
/* Write out a bitmap...
  * Current only supporting 8 bits per pixel...
  * @String filename - The file to save it as...
  * @IndexColorModel ICM - Palette to use 
  * @int pix[] - Pixels to save
  * @int width - Width of data
 * @int height - Height of data
  */
static void saveBitmap(String filename, Component comp) {
	icm = null;
	try {
		Dimension dim = comp.getSize();
		int width = dim.width;
		int height = dim.height;
		Image image = comp.createImage(dim.width, dim.height);

		Graphics g = image.getGraphics();
		g.setColor(Color.white);
		g.fillRect(0, 0, dim.width, dim.height);
		g.setColor(Color.black);
		comp.paint(g);
		
		int pix[] = new int[width * height];
		PixelGrabber pg = new PixelGrabber(image, 0, 0, width, height, pix, 0, width);
		pg.grabPixels();
		if ((pg.getStatus() & ImageObserver.ABORT) != 0) {
			System.err.println("image fetch aborted or errored");
			return;
		}
		_mapColours(pix);

		DataOutputStream os = new DataOutputStream(new BufferedOutputStream(new FileOutputStream(filename)));
		_writeFileHeader(os, width, height);
		_write8bitWindowsHeader(os, width, height);
		_write8bitColorIndex(os);

		_write8bitData(os, pix, width, height);
		os.close();
		} catch (InterruptedException e) {
			// probably from pixel grabber
			System.out.println(e);
			return;
	} catch (IOException e) {
		System.out.println(e);
		e.printStackTrace();
	}
}
/* Write out a bitmap...
  * Current only supporting 8 bits per pixel...
  * @String filename - The file to save it as...
  * @IndexColorModel ICM - Palette to use 
  * @int pix[] - Pixels to save
  * @int width - Width of data
 * @int height - Height of data
  */
static void saveBitmap(String filename, Image image) {
	icm = null; 
	try {
		int width = image.getWidth(null);
		int height = image.getHeight(null);

//		Graphics g = image.getGraphics();
//		g.setColor(Color.white);
//		g.fillRect(0, 0, dim.width, dim.height);
//		g.setColor(Color.black);
//		comp.paint(g);
		
		int pix[] = new int[width * height];
		PixelGrabber pg = new PixelGrabber(image, 0, 0, width, height, pix, 0, width);
		pg.grabPixels();
		if ((pg.getStatus() & ImageObserver.ABORT) != 0) {
			System.err.println("image fetch aborted or errored");
			return;
		}
		_mapColours(pix);

		DataOutputStream os = new DataOutputStream(new BufferedOutputStream(new FileOutputStream(filename)));
		_writeFileHeader(os, width, height);
		_write8bitWindowsHeader(os, width, height);
		_write8bitColorIndex(os);

		_write8bitData(os, pix, width, height);
		os.close();
		} catch (InterruptedException e) {
			// probably from pixel grabber
			System.out.println(e);
			return;
	} catch (IOException e) {
		System.out.println(e);
		e.printStackTrace();
	}
}
}
