package glyph;

import java.io.*;
import java.awt.Color;
import java.awt.Font;
/**
 * Insert the type's description here.
 * Creation date: (08/08/2002 08:02:50)
 * @author: 
 */
public class MetafileContext extends DrawContext {
    private RandomAccessFile out;
    private int byteCount = 0;
    private int maxRecordSize = 3; 		// end record is three words long
    private int recordCount = 100; 		// at least the end record
    private boolean disposed = false;	// make sure we dispose only once
    private int maxObjectCount = 0;

    private static final int MAGIC = 0x9AC6CDD7;	// indicates placeble metafile
    private static final int TWIPS = 100;			// sacling factor
	private static final int MOVETO = 0x0214;
	private static final int LINETO = 0x0213;
	private static final int FILLRECT = 0x041B;
	private static final int DRAWTEXT = 0x0521;
	private static final int SETTEXTCOLOR = 0x209;

	private int blackPen = 2;
	private int whitePen = 3;
	private int lightGreyPen = 4;

	private int blackBrush = 5;
	private int whiteBrush = 6;
	private int lightGreyBrush = 7;

	private int dottedPen = 8;
	

    private int width = 10000;
    private int height = 10000;
/**
 * Metafile constructor comment.
 */
protected MetafileContext(String outFileName, int screenWidth, int screenHeight, Font font) {
    super();
    width = screenWidth;
    height = screenHeight;
    try {
        File file = new File(outFileName);
        if (!file.exists()) {
	        //System.out.println("creating file");
            file.createNewFile();
        }
        //System.out.println("opening file");
        out = new RandomAccessFile(outFileName, "rw");
        // write the header.  We do not have the information, 
        // but we can overwite this when we close
        //System.out.println("writing initial header");
		_writeHeader(0, 0, 0);
        //System.out.println("written header");

        //_setMapMode();
        _setBkMode(1);
        _setTextAlign(24);
        _setWindowOrg(0,0);
        _setWindowExt(width, height);
        _createFont(font);
        _selectObject(0);
        _createPallette();
        
        //_setTextColor(0x00808000);
        blackBrush = _createBrush(0);  				//black brush (object #2)
        whiteBrush = _createBrush(0x00ffffff);  		//white brush (object #3)
        lightGreyBrush= _createBrush(0x00c0c0c0);  	//light grey brush (object #4)
        
        blackPen = _createPen(0, 0);					// black pen object #5
        dottedPen = _createPen(0, 2);
        whitePen = _createPen(0x00ffffff, 0);					// white pen object #6
        lightGreyPen = _createPen(0x00c0c0c0, 0);				// light grey pen object #7
        
        
    } catch (IOException e) {
        e.printStackTrace();
    }
}
private final int _createBrush(int colour) {
	_pushDword(7);
	_pushWord(0x02FC);	
	_pushWord(0);	//style
	_pushDword(colour);	//colour
	_pushWord(0);	// hatch

	if (maxRecordSize < 7) {
		maxRecordSize = 7;
	} 
    recordCount++;

    return ++maxObjectCount;
}
private final void _createFont(Font font) {
	// prepare fontName as bytes
	String fontName = font.getName();
	final int strLen = fontName.length();
	int stringbyteBufLen = strLen+1;
	if ((stringbyteBufLen & 1) == 1)  { // round up odd bytes to make buffer word aligned
		stringbyteBufLen += 1;
	}
	byte[] fontNameBytes = new byte[stringbyteBufLen];
	fontName.getBytes(0, strLen, fontNameBytes, 0);
	fontNameBytes[strLen] = 0;

	// now the writing
	final int recordSize = 12+ (stringbyteBufLen/2); 

	_pushDword(recordSize);
	_pushWord(0x02FB);	// set font
	_pushWord(font.getSize());		// logical height
	_pushWord(0);		// width
	_pushWord(0);		// tilt
	_pushWord(0);		// orientation
	_pushWord(700);	    // emphasis - normal
	_pushWord(0x0000);	// no italic, no underline
	_pushWord(0x0000);	// ansii charset, no stikeout 
	_pushWord(0x0000);	// precision
	_pushWord(0x0000);  // pitch and family
	try {
		out.write(fontNameBytes);
		byteCount += fontNameBytes.length;
	} catch (IOException e) {
		e.printStackTrace();
	}

	if (maxRecordSize < recordSize) {
		maxRecordSize = recordSize;
	} 
    recordCount++;
    maxObjectCount++;
}
private final void _createPallette() {
	_pushDword(8);
	_pushWord(0x00F7);	
	_pushWord(0x0300);	//version
	_pushWord(3);	//three colours
//	_pushWord(0);	// black
	_pushWord(0x00ffffff);	// white
//	_pushWord(0x00c0c0c0);	// light grey
	_pushWord(0x00ffffff);	// white
	_pushWord(0x00ffffff);	// white

	if (maxRecordSize < 8) {
		maxRecordSize = 8;
	} 
    recordCount++;

    _pushDword(3);
    _pushWord(0x035);  // realize pallette ?
    //_pushWord(PALLETTE);
    recordCount++;
}
private final int _createPen(int colour,int style) {
	_pushDword(7);
	_pushWord(0x02FA);
	// 5 = no line....
	// 4 = dot dot dash dash
	// 3 = dot dash
	// 2 = dot dot dot
	// 1 = dash dash dash
	// 0 = solid
	if (0 != colour) {
		// for white overide the line style
		style = 5;
		colour = 0;
	}	
	_pushWord(style);	//style
	_pushWord(0);	//width
	_pushWord(0);	// unused
	_pushWord(colour);	// color index

	if (maxRecordSize < 7) {
		maxRecordSize = 7;
	} 
    recordCount++;
    return ++maxObjectCount;
}
	/**
	 * text is always drawn in black
	 */
private final void _drawString(String str, int x, int y) {
	final int strLen = str.length();
	int stringbyteBufLen = strLen+1;
	if ((stringbyteBufLen & 1) == 1)  { // round up odd bytes to make buffer word aligned
		stringbyteBufLen += 1;
	}
	final int recordSize = 6+ (stringbyteBufLen/2); 
	_pushDword(recordSize);			// recordSize (words)
	_pushWord(DRAWTEXT);		// record type
	_pushWord(strLen);	// string length
	try {
		
		byte[] bytes = new byte[stringbyteBufLen];
		str.getBytes(0, str.length(), bytes, 0);
		bytes[strLen] = 0;
		out.write(bytes);
		byteCount += bytes.length;
	} catch (IOException e) {
		e.printStackTrace();
	}
	_pushWord(y);			// yCoord
	_pushWord(x);			// xCoord
	
	if (maxRecordSize < recordSize) {
		maxRecordSize = recordSize;
	}

	recordCount +=1;
	
}
private final void _lineTo(int x, int y) {
	
	_pushDword(5);			// recordSize (words)
	_pushWord(LINETO);		// record type
	_pushWord(y);			// yCoord
	_pushWord(x);			// xCoord

	if (maxRecordSize < 5) {
		maxRecordSize = 5;
	}

	recordCount += 1;
}
private final void _moveTo(int x, int y) {
	_pushDword(5);			// recordSize (words)
	_pushWord(MOVETO);		// record type
	_pushWord(y);			// yCoord
	_pushWord(x);			// xCoord
	
	if (maxRecordSize < 5) {
		maxRecordSize = 5;
	}

	recordCount += 1;
}
private final void _polygon(int[] xPoints, int[] yPoints, int nPoints) {
	int recordSize = 4 + (2*nPoints);
	_pushDword(recordSize);
	_pushWord(0x0324);	
	_pushWord(nPoints);
	for (int i = 0; i < nPoints; i++){
		_pushWord(xPoints[i]);
		_pushWord(yPoints[i]);
	}

	if (maxRecordSize < recordSize) {
		maxRecordSize = recordSize;
	} 
    recordCount++;
}
	private final boolean _pushDword(int i) {
		return _pushVal(i, 4);
	}
/**
  * Write out integer to little endian stream...
  * @DataOutputStream os - The output stream to write
  * @int data - Data to convert
  * @int len - Length of array to convert 
  */
private final boolean _pushVal(int data, int len) {
	boolean success = true;
	try {
		byte b[] = new byte[len];
		for (int i = 0; i < len; ++i) {
			b[i] = (byte) (data >> (i * 8));
		}
		out.write(b, 0, len);
		byteCount += len;
	} catch (IOException e) {
		e.printStackTrace();
		success = false;
	}
	return success;
}
	private final boolean _pushWord(int i) {
		return _pushVal(i, 2);
	}
private final void _rectangle(int x1, int y1, int x2, int y2) {
	_pushDword(7);
	_pushWord(0x041B);	
	_pushWord(y1);
	_pushWord(x1);
	_pushWord(y2);
	_pushWord(x2);

	if (maxRecordSize < 7) {
		maxRecordSize = 7;
	} 
    recordCount++;
}
private final void _selectObject(int i) {
	_pushDword(4);
	_pushWord(0x012D);	// select object
	_pushWord(i);		// font we just created?
    recordCount++;
}
private final void _setBkMode(int mode) {
	_pushDword(4);
	_pushWord(0x0102);	// set mapmode
	_pushWord(mode);		// dont ask me!

	if (maxRecordSize < 4) {
		maxRecordSize = 4;
	} 
    recordCount++;
}
private final void _setMapMode() {
	_pushDword(4);
	_pushWord(0x0103);	// set mapmode
	_pushWord(8);		// dont ask me!

	if (maxRecordSize < 4) {
		maxRecordSize = 4;
	} 
    recordCount++;
}
private final void _setPixel(int x, int y, int color) {
	_pushDword(7);			// recordSize (words)
	_pushWord(0x041F);		// record type
	_pushWord(y);		
	_pushWord(x);		
	_pushDword(color);		

	if (maxRecordSize < 7) {
		maxRecordSize = 7;
	} 
    recordCount++;
}
private final void _setTextAlign(int align) {
	_pushDword(4);			// recordSize (words)
	_pushWord(0x012E);		// record type
	_pushWord(align);		

	if (maxRecordSize < 4) {
		maxRecordSize = 4;
	} 
    recordCount++;
}
private final void _setTextColor(int color) {
	_pushDword(5);			// recordSize (words)
	_pushWord(SETTEXTCOLOR);		// record type
	_pushDword(color);			// colour

	if (maxRecordSize < 5) {
		maxRecordSize = 5;
	} 
    recordCount++;
}
private final void _setWindowExt(int x, int y) {
	_pushDword(5);
	_pushWord(0x020C);	
	_pushWord(y);
	_pushWord(x);

	if (maxRecordSize < 5) {
		maxRecordSize = 5;
	} 
    recordCount++;
}
private final void _setWindowOrg(int x, int y) {
	_pushDword(5);
	_pushWord(0x020B);	
	_pushWord(y);
	_pushWord(x);

	if (maxRecordSize < 5) {
		maxRecordSize = 5;
	} 
    recordCount++;
}
private final boolean _writeHeader(int wordCount, int objectCount, int maxElementSize) {
    //System.out.println("header(wordCount=" + wordCount + ", elemCOunt=" + elemCount + ", maxSize=" + maxElementSize + ")");
    try {
        out.seek(0);
    } catch (Exception e) {
        e.printStackTrace();
    }
    boolean success = true;
    if (true) {

        int check = MAGIC & 0xFFFF;
        check ^= (MAGIC & 0xFFFF0000);

        _pushDword(MAGIC); // magic number
        _pushWord(0); // handle always zero
        _pushWord(0); // left
        _pushWord(0); // top
        _pushWord(width); // right
        check ^= (width);
        _pushWord(height); // bottom
        check ^= (height);
        _pushWord(TWIPS); // twips
        check ^= TWIPS;
        _pushDword(0); //reserved
        _pushWord(check); //reserved
        byteCount = 0;		// placeable header does not count in byteLenght
    }
    
    if (true) {
        success = _pushWord(1); // file representation
        if (success) {
            success = _pushWord(9); // header length in words
        }
        if (success) {
            success = _pushWord(0x300); // Version 3 onwards....
        }
        if (success) {
            success = _pushDword(wordCount); // file length in words
        }
        if (success) {
            success = _pushWord(objectCount); // number of objects
        }
        if (success) {
            success = _pushDword(maxElementSize); // maximum size of an element
        }
        if (success) {
            success = _pushWord(0); // unused
        }
    }
    return success;
}
/**
 * Disposes of this graphics context and releases 
 * any system resources that it is using. 
 * A <code>Graphics</code> object cannot be used after 
 * <code>dispose</code>has been called.
 * <p>
 * When a Java program runs, a large number of <code>Graphics</code>
 * objects can be created within a short time frame.
 * Although the finalization process of the garbage collector 
 * also disposes of the same system resources, it is preferable 
 * to manually free the associated resources by calling this
 * method rather than to rely on a finalization process which 
 * may not run to completion for a long period of time.
 * <p>
 * Graphics objects which are provided as arguments to the 
 * <code>paint</code> and <code>update</code> methods 
 * of components are automatically released by the system when 
 * those methods return. For efficiency, programmers should
 * call <code>dispose</code> when finished using
 * a <code>Graphics</code> object only if it was created 
 * directly from a component or another <code>Graphics</code> object.
 * @see         java.awt.Graphics#finalize
 * @see         java.awt.Component#paint
 * @see         java.awt.Component#update
 * @see         java.awt.Component#getGraphics
 * @see         java.awt.Graphics#create
 */
public void dispose() {
    if (!disposed) {
	    
	    disposed = true;
        //System.out.println("Dispose - creating end record");
        final int ENDRECORD = 0x0000;
        _pushDword(3); // recordSize (words)
        _pushWord(ENDRECORD); // record type
        //System.out.println("Dispose - rewriting header (bytes="+byteCount+")");
        
		_writeHeader(byteCount / 2, maxObjectCount, maxRecordSize);
        try {
            //System.out.println("closing file");
            out.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
	/**
	 * lines are always drawn in black
	 */
void drawDottedLine(int x1, int y1, int x2, int y2) {
	_selectObject(dottedPen);
	drawLine(x1,y1,x2,y2);
	_selectObject(blackPen);
}
	/**
	 * lines are always drawn in black
	 */
void drawLine(int x1, int y1, int x2, int y2) {
//	_setPixel(x1,y1,0); //draw start pixel
	_moveTo(x1, y1);
	_lineTo(x2, y2);
	_setPixel(x2,y2,0); //draw end pixel
}
/**
 * drawPolygon method comment.
 */
void drawPolygon(int[] x, int[] y, int nPoints, Color border, Color fill) {
	if (Color.black.equals(border)) {
		_selectObject(blackPen);
	} else if (Color.white.equals(border)) {
		_selectObject(whitePen);
	} else if (Color.lightGray.equals(border)) {
		_selectObject(lightGreyPen);
	}
	if (Color.black.equals(fill)) {
		_selectObject(blackBrush);
	} else if (Color.white.equals(fill)) {
		_selectObject(whiteBrush);
	} else if (Color.lightGray.equals(fill)) {
		_selectObject(lightGreyBrush);
	}
	_polygon(x, y, nPoints);
}
	/**
	 * text is always drawn in black
	 */
void drawString(String text, int x, int y) {
	_drawString(text, x, y);
}
/**
 * Insert the method's description here.
 * Creation date: (29/07/2002 09:31:46)
 * @param args java.lang.String[]
 */
public static void main(String[] args) {
	Font font = new Font("Helvetica", Font.PLAIN, 12);
	MetafileContext wmf = new MetafileContext("f:\\temp\\diagram.wmf", 50,50, font);
	//wmf.drawString("Test Text", 100,100);
	
	wmf.drawLine(10,10,10,20);
	wmf.drawLine(10,20,20,20);
	wmf.drawLine(20,20,20,10);
	wmf.drawLine(20,10,10,10);
	
	wmf.dispose();
}
}
